@@ -31,7 +31,7 @@
 
 
 /*
- * $Id: dlsof.h,v 1.8 98/03/06 08:04:29 abe Exp $
+ * $Id: dlsof.h,v 1.18 2000/03/14 20:34:27 abe Exp $
  */
 
 
@@ -50,11 +50,17 @@
 
 #include <sys/vnode.h>
 #include <sys/file.h>
-#include <procinfo.h>
+#include <procinfo.h>		/* <procinfo.h> #includes <sys/user.h> */
 #define p_pid	pi_pid
 #define	p_pgrp	pi_pgrp
 #define	p_ppid	pi_ppid
+
+# if	AIXV<4300
 #define p_stat	pi_stat
+# else	/* AIXV>=4300 */
+#define	p_stat	pi_state
+# endif	/* AIXV<4300 */
+
 #define p_uid	pi_uid
 #undef	sleep
 #undef	_KERNEL
@@ -87,6 +93,11 @@
 
 #include <rpc/rpc.h>
 #include <rpc/pmap_prot.h>
+
+# if	defined(HAS_AFS)
+#define	__XDR_INCLUDE__
+# endif	/* defined(HAS_AFS) */
+
 #include <sys/vfs.h>
 #include <sys/vmount.h>
 
@@ -110,8 +121,33 @@
 # endif	/* AIXV>=4100 */
 
 # if	defined(HAS_NFS)
+#  if	AIXV<4210
 #include <nfs/rnode.h>
-# endif	/* defined(HAS_NFS) */
+#  else	/* AIXV>=4210 */
+/*
+ * Private rnode struct definition for AIX 4.2.1 and above
+ *
+ * The rnode struct IBM ships in <nfs/rnode.h> doesn't match the one
+ * the kernel uses.  The kernel's rnode struct definition comes from
+ * <oncplus/nfs/rnode.h>, a header file IBM does not ship with AIX.
+ */
+
+struct rnode {
+	caddr_t	r_d1[11];		/* dummies; links? */
+	struct vnode r_vnode;		/* vnode for remote file */
+	struct gnode r_gnode;		/* gnode for remote file */
+
+#  if	AIXV<4330
+	caddr_t r_d2[29];		/* dummies; rnode elements? */
+#  else	/* AIXV>=4330 */
+	caddr_t r_d2[26];		/* dummies; rnode elements? */
+#  endif	/* AIXV<4330 */
+
+	off_t r_size;			/* client's view of file size (long)*/
+	struct vattr r_attr;		/* cached vnode attributes */
+};
+#  endif	/* AIXV<4210 */
+# endif	/* defined(HAS_NFS) && AIXV<4210 */
 
 #  if	AIXV>=3200
 #include <sys/specnode.h>
@@ -134,55 +170,68 @@ struct cdrnode {
 	off_t		cn_size;	/* size of file in bytes */
 };
 
+
 /*
  * AIX doesn't supply ld_data.h.
  */
 
-struct le {
+struct le {				/* loader entry */
+
 	struct le *next;		/* next entry pointer */
+
+# if	AIXV<4300
 	ushort dummy1;
 	ushort dummy2;
 	uint dummy3;
 	struct file *fp;		/* file table entry pointer */
 
-# if	AIXV>=4110
+#  if	AIXV>=4110
 	int ft;				/* file type indicator */
 	unsigned dummy4;
 	char *dummy5;
 	unsigned dummy6;
 	char *dummy7[3];
 	char *nm;			/* name */
-# endif	/* AIXV>=4110 */
+#  endif	/* AIXV>=4110 */
+# else	/* AIXV>=4300 */
+	uint flags;
+	struct file *fp;		/* file table entry pointer */
+	char *nm;			/* name */
+# endif	/* AIXV<4300 */
 
 };
 
+# if	AIXV>=4300
+/*
+ * The elements of interest from the AIX >= 4.3 loader anchor structure.
+ */
+struct la {			/* loader anchor */
+
+    struct le *list;
+    struct le *exec;
+};
+# endif	/* AIXV>=4300 */
+
 # if	defined(HAS_AFS)
+
+/*
+ *  Avoid typdef conflicts in <afs/stds.h>.
+ */
+
+#  if	defined(HASINT16TYPE)
+#define	int16	AFS_int16
+#  endif	/* defined(HASINT16TYPE */
+#  if	defined(HASUINT16TYPE)
+#define	u_int16	AFS_u_int16
+#  endif	/* defined(HASUINT16TYPE */
+#  if	defined(HASINT32TYPE)
+#define int32	AFS_int32
+#  endif	/* defined(HASINT32TYPE) */
+
 #include <afs/stds.h>
 #include <afs/param.h>
 #include <afs/afsint.h>
 #include <afs/vldbint.h>
-
-
-/*
- * This is an emulation of the afs_rwlock_t definition that appears in
- * the AFS sources in afs/lock.h.
- */
-
-struct afs_lock {
-
-#  if	HAS_AFS<304
-    unsigned long d1[4];
-#  else	/* HAS_AFS>=304 */
-    unsigned long d1[6];
-#  endif	/* HAS_AFS<304 */
-
-};
-typedef struct afs_lock afs_lock_t;
-typedef struct afs_lock afs_rwlock_t;
-
-#define	KERNEL
-#include <afs/afs.h>
-#undef	KERNEL
 # endif	/* defined(HAS_AFS) */
 
 
@@ -201,13 +250,18 @@ typedef struct afs_lock afs_rwlock_t;
 
 #define	COMP_P		const void
 #define DEVINCR		1024	/* device table malloc() increment */
-#define	KA_T		off_t
+
+# if	AIXV<4200
+typedef	off_t		KA_T;
+# else	/* AIXV>=4200 */
+typedef	unsigned int	KA_T;
+# endif	/* AIXV<4200 */
+
 #define KMEM		"/dev/kmem"
 #define MALLOC_P	char
 #define FREE_P		MALLOC_P
 #define MALLOC_S	unsigned
 #define	N_UNIX		"/unix"
-#define PROCSIZE	sizeof(struct procinfo)
 #define QSORT_P		void
 #define	READLEN_T	unsigned
 #define STRNCPY_L	size_t
@@ -244,7 +298,7 @@ typedef struct afs_lock afs_rwlock_t;
 
 
 /*
- * Define for ckfa.c
+ * Definition for ckfa.c
  */
 
 #define	CKFA_MPXCHAN	1
@@ -254,12 +308,20 @@ typedef struct afs_lock afs_rwlock_t;
  * Definitions for dvch.c
  */
 
-#if	AIXV>=4140
+# if	AIXV>=4140
 #define	DCACHE_CLONE	rw_clone_sect	/* clone function for read_dcache */
 #define	DCACHE_CLR	clr_sect	/* function to clear clone and
 					 * pseudo caches when reading the
 					 * device cache file fails */
-#endif	/* AIXV>=4140 */
+# endif	/* AIXV>=4140 */
+
+
+/*
+ * Definitions for enter_dir()
+ */
+
+#define	DIRTYPE	dirent
+#define	HASDNAMLEN	1
 
 
 # if	defined(HAS_AFS)
@@ -355,11 +417,24 @@ extern struct nlist AFSnl[];	/* AFS kernel symbol name list table */
 extern char *AFSApath;		/* alternate AFS name list path (from -a) */
 #  endif	/* defined(HASAOPT) */
 
-KA_T AFSVfsp;			/* AFS struct vfs kernel pointer */
+extern KA_T AFSVfsp;		/* AFS struct vfs kernel pointer */
 # endif	/* defined(HAS_AFS) */
 
 extern int Kd;
 extern int Km;
 extern struct nlist Nl[];
 
+# if	defined(TCPSTATES) && AIXV<=3250
+/*
+ * For AIX 3.2.5 and below, there is no header file with the  definition
+ * of tcpstates[], needed by ptti.c's print_tcptpi() function.
+ */
+
+static char *tcpstates[] = {
+	"CLOSED",       "LISTEN",       "SYN_SENT",     "SYN_RCVD",
+	"ESTABLISHED",  "CLOSE_WAIT",   "FIN_WAIT_1",   "CLOSING",
+	"LAST_ACK",     "FIN_WAIT_2",   "TIME_WAIT"
+};
+# endif	/* defined(TCPSTATES) && AIXV<=3250 */
+
 #endif	/* AIX_LSOF_H */
