@@ -32,7 +32,7 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dsock.c,v 1.5 98/03/06 08:04:38 abe Exp $";
+static char *rcsid = "$Id: dsock.c,v 1.12 2000/03/14 20:34:27 abe Exp $";
 #endif
 
 
@@ -41,11 +41,12 @@ static char *rcsid = "$Id: dsock.c,v 1.5 98/03/06 08:04:38 abe Exp $";
 
 /*
  * We include <sys/domain.h> here instead of "dlsof.h" for gcc's benefit.
- * It's loader can't handle the multiple CONST u_char arrays declared in
+ * Its loader can't handle the multiple CONST u_char arrays declared in
  * <net/net_globals.h> -- e.g., etherbroadcastaddr[].  (<sys/domain.h>
  * #include's <net/net_globals.h>.)
  */
 
+#include <net/netopt.h>
 #include <sys/domain.h>
 
 
@@ -58,25 +59,22 @@ process_socket(sa)
 	KA_T sa;			/* socket address in kernel */
 {
 	struct domain d;
-	struct in_addr *fa = (struct in_addr *)NULL;
+	unsigned char *fa = (unsigned char *)NULL;
 	int fam;
 	int fp, lp;
 	struct gnode g;
 	struct inode i;
 	struct inpcb inp;
 	int is = 0;
-	struct in_addr *la = (struct in_addr *)NULL;
+	unsigned char *la = (unsigned char *)NULL;
 	struct mbuf mb;
 	struct protosw p;
-	struct rawcb raw;
 	struct socket s;
-	struct sockaddr sad;
 	struct tcpcb t;
 	struct unpcb uc, unp;
-	struct sockaddr_un *ua = NULL;
+	struct sockaddr_un *ua = (struct sockaddr_un *)NULL;
 	struct sockaddr_un un;
 	struct vnode v;
-
 /*
  * Set socket file variables.
  */
@@ -100,8 +98,10 @@ process_socket(sa)
 	    return;
 	}
 	if (!s.so_proto
-	||  kread((KA_T) s.so_proto, (char *) &p, sizeof(p))) {
-	    enter_nm("no protocol switch");
+	||  kread((KA_T)s.so_proto, (char *)&p, sizeof(p))) {
+	    (void) sprintf(Namech, "can't read protocol switch from %s",
+		print_kptr((KA_T)s.so_proto, (char *)NULL));
+	    enter_nm(Namech);
 	    return;
 	}
 /*
@@ -127,89 +127,131 @@ process_socket(sa)
 /*
  * Process socket by the associated domain family.
  */
-	if (kread((KA_T) p.pr_domain, (char *) &d, sizeof(d))) {
+	if (!p.pr_domain
+	||  kread((KA_T)p.pr_domain, (char *)&d, sizeof(d))) {
 	    (void) sprintf(Namech, "can't read domain struct from %s",
 		print_kptr((KA_T)p.pr_domain, (char *)NULL));
 	    enter_nm(Namech);
 	    return;
 	}
-
 	switch ((fam = d.dom_family)) {
 /*
  * Process an Internet domain socket.
  */
 	case AF_INET:
+
+#if	defined(HASIPv6)
+	case AF_INET6:
+#endif	/* defined(HASIPv6) */
+
 	    if (Fnet)
 		Lf->sf |= SELNET;
-	    (void) strcpy(Lf->type, "inet");
 	    printiproto(p.pr_protocol);
+
+#if	defined(HASIPv6)
+	    (void) strcpy(Lf->type, fam == AF_INET ? "IPv4" : "IPv6");
+#else	/* !defined(HASIPv6) */
+	    (void) strcpy(Lf->type, "inet");
+#endif	/* defined(HASIPv6) */
+
 	/*
 	 * Read protocol control block.
 	 */
-	    if (!s.so_pcb) {
-		enter_nm("no protocol control block");
-		return;
-	    }
-	    if (s.so_type == SOCK_RAW) {
-
-	    /*
-	     * Print raw socket information.
-	     */
-		if (kread((KA_T)s.so_pcb, (char *)&raw, sizeof(raw))
-		||  (struct socket *)sa != raw.rcb_socket) {
-		    (void) sprintf(Namech, "can't read rawcb at %s",
-			print_kptr((KA_T)s.so_pcb, (char *)NULL));
-		    enter_nm(Namech);
-		    return;
-		}
-		enter_dev_ch(print_kptr((KA_T)s.so_pcb, (char *)NULL));
-		if (raw.rcb_laddr
-		&& !kread((KA_T)raw.rcb_laddr, (char *)&sad, sizeof(sad))) {
-		    if (sad.sa_family == AF_INET)
-			la = (struct in_addr *)&sad.sa_data[2];
-		    else if (sad.sa_family)
-			printrawaddr(&sad);
-		}
-		if (raw.rcb_laddr
-		&& !kread((KA_T)raw.rcb_faddr, (char *)&sad, sizeof(sad))) {
-		    if (sad.sa_family == AF_INET)
-			fa = (struct in_addr *)&sad.sa_data[2];
-		    else if (sad.sa_family) {
-			(void) strcat(endnm(), "->");
-			printrawaddr(&sad);
-		    }
-		}
-		if (fa || la)
-		    (void) ent_inaddr(la, -1, fa, -1);
-	    } else {
-
-	    /*
-	     * Save Internet socket information.
-	     */
-		if (kread((KA_T) s.so_pcb, (char *) &inp, sizeof(inp))
-		||  (struct socket *)sa != inp.inp_socket) {
+	    if (!s.so_pcb
+	    ||  kread((KA_T) s.so_pcb, (char *) &inp, sizeof(inp))) {
+		if (!s.so_pcb) {
+		    (void) strcpy(Namech, "no PCB");
+		    if (s.so_state & SS_CANTSENDMORE)
+			(void) strcpy(endnm(), ", CANTSENDMORE");
+		    if (s.so_state & SS_CANTRCVMORE)
+			(void) strcpy(endnm(), ", CANTRCVMORE");
+		} else {
 		    (void) sprintf(Namech, "can't read inpcb at %s",
 			print_kptr((KA_T)s.so_pcb, (char *)NULL));
-		    enter_nm(Namech);
-		    return;
 		}
-		enter_dev_ch(print_kptr((KA_T)(inp.inp_ppcb ? inp.inp_ppcb
-							    : s.so_pcb),
-					(char *)NULL));
-		la = &inp.inp_laddr;
-		lp = (int)ntohs(inp.inp_lport);
-		if (inp.inp_faddr.s_addr != INADDR_ANY || inp.inp_fport != 0) {
-		    fa =  &inp.inp_faddr;
-		    fp = (int)ntohs(inp.inp_fport);
-		}
-		if (fa || la)
-		    (void) ent_inaddr(la, lp, fa, fp);
-		if (p.pr_protocol == IPPROTO_TCP && inp.inp_ppcb
-		&&  !kread((KA_T)inp.inp_ppcb, (char *)&t, sizeof(t))) {
-		    Lf->lts.type = 0;
-		    Lf->lts.state.i = (int)t.t_state;
+		enter_nm(Namech);
+		return;
+	    }
+	/*
+	 * Save Internet socket information.
+	 */
+	    enter_dev_ch(print_kptr((KA_T)(inp.inp_ppcb ? inp.inp_ppcb
+							: s.so_pcb),
+				    (char *)NULL));
+
+#if	defined(HASIPv6)
+	/*
+	 * If this is an IPv6 (AF_INET6) socket and IPv4 compatibility
+	 * mode is enabled, use the IPv4 address, change the family
+	 * indicator from AF_INET6 to AF_INET, and change the file type
+	 * from "IPv6" to "IPv4".  Othwerise, use the IPv6 address.
+	 * Don't ignore empty addresses.
+	 */
+	    if (fam == AF_INET6) {
+		if (inp.inp_flags & INP_COMPATV4) {
+		    Lf->type[3] = '4';
+		    fam = AF_INET;
+		    la = (unsigned char *)&inp.inp_laddr;
+		} else
+		    la = (unsigned char *)&inp.inp_laddr6;
+	    } else
+#endif	/* defined(HASIPv6) */
+
+		la = (unsigned char *)&inp.inp_laddr;
+	    lp = (int)ntohs(inp.inp_lport);
+	    if (fam == AF_INET
+	    &&  (inp.inp_faddr.s_addr != INADDR_ANY || inp.inp_fport != 0)) {
+		fa =  (unsigned char *)&inp.inp_faddr;
+		fp = (int)ntohs(inp.inp_fport);
+	    }
+
+#if	defined(HASIPv6)
+	    else if (fam == AF_INET6) {
+
+	    /*
+	     * If this is an IPv6 (AF_INET6) socket and IPv4 compatibility
+	     * mode is enabled, use the IPv4 address, change the family
+	     * indicator from AF_INET6 to AF_INET, and change the file type
+	     * from "IPv6" to "IPv4".  Othwerise, use the IPv6 address.
+	     * Ignore empty addresses.
+	     */
+		if (inp.inp_flags & INP_COMPATV4) {
+		    Lf->type[3] = '4';
+		    fam = AF_INET;
+		    if (inp.inp_faddr.s_addr != INADDR_ANY
+		    || inp.inp_fport != 0)
+		    {
+			fa = (unsigned char *)&inp.inp_faddr;
+			fp = (int)ntohs(inp.inp_fport);
+		    }
+		} else {
+		    if (!IN6_IS_ADDR_UNSPECIFIED(&inp.inp_faddr6)) {
+			fa = (unsigned char *)&inp.inp_faddr6;
+			fp = (int)ntohs(inp.inp_fport);
+		    }
 		}
 	    }
+#endif	/* defined(HASIPv6) */
+
+	    if (fa || la)
+		(void) ent_inaddr(la, lp, fa, fp, fam);
+	    if (p.pr_protocol == IPPROTO_TCP && inp.inp_ppcb
+	    &&  !kread((KA_T)inp.inp_ppcb, (char *)&t, sizeof(t))) {
+		Lf->lts.type = 0;
+		Lf->lts.state.i = (int)t.t_state;
+	    }
+	    break;
+/*
+ * Process a ROUTE domain socket.
+ */
+	case AF_ROUTE:
+	    (void) strcpy(Lf->type, "rte");
+	    if (s.so_pcb)
+		enter_dev_ch(print_kptr((KA_T)(s.so_pcb), (char *)NULL));
+	    else
+		(void) strcpy(Namech, "no protocol control block");
+	    if (!Fsize)
+		Lf->off_def = 1;
 	    break;
 /*
  * Process a Unix domain socket.
@@ -308,14 +350,14 @@ process_socket(sa)
 		*((char *)ua + mb.m_len) = '\0';
 		if (Sfile && is_file_named(ua->sun_path, VSOCK, 0, 0))
 		    Lf->sf |= SELNM;
-		else
+		if (!Namech[0])
 		    (void) strcpy(Namech, ua->sun_path);
 	    } else
 		(void) strcpy(Namech, "no address");
 	    break;
 
 	default:
-	    printunkaf(fam);
+	    printunkaf(fam, 1);
 	}
 	if (Namech[0])
 	    enter_nm(Namech);
