@@ -31,7 +31,7 @@
 
 
 /*
- * $Id: dlsof.h,v 1.8 98/02/13 12:43:11 abe Exp $
+ * $Id: dlsof.h,v 1.12 2000/03/23 14:21:02 abe Exp $
  */
 
 
@@ -45,6 +45,11 @@
 #include <signal.h>
 #include <string.h>
 #include <unistd.h>
+
+# if	BSDIV>=40100
+#include <arpa/inet.h>
+# endif	/* BSDIV>=40100 */
+
 #include <sys/filedesc.h>
 #include <sys/mbuf.h>
 #define	NFS
@@ -57,6 +62,11 @@
 #include <sys/un.h>
 #include <sys/unpcb.h>
 #include <netinet/in.h>
+
+# if	defined(HASIPv6) && !defined(IN6_ARE_ADDR_EQUAL) && defined(IN6_IS_ADDR_EQUAL)
+#define	IN6_ARE_ADDR_EQUAL	IN6_IS_ADDR_EQUAL
+# endif	/* defined(HASIPv6) && !defined(IN6_ARE_ADDR_EQUAL) && defined(IN6_IS_ADDR_EQUAL) */
+
 #include <netinet/in_systm.h>
 #include <netinet/ip.h>
 #include <net/route.h>
@@ -151,7 +161,7 @@
 
 #define	COMP_P		const void
 #define DEVINCR		1024	/* device table malloc() increment */
-#define	KA_T		u_long
+typedef	u_long		KA_T;
 #define	KMEM		"/dev/kmem"
 #define MALLOC_P	void
 #define FREE_P		MALLOC_P
@@ -167,12 +177,12 @@
  * Global storage definitions (including their structure definitions)
  */
 
-struct file *Cfp;
+KA_T Cfp;
 extern kvm_t *Kd;
 extern KA_T Kpa;
 
 struct l_vfs {
-	struct mount *addr;		/* kernel address */
+	KA_T addr;			/* kernel address */
 	fsid_t	fsid;			/* file system ID */
 
 #if	BSDIV<30000
