@@ -31,7 +31,7 @@
 
 
 /*
- * $Id: dlsof.h,v 1.7 98/02/13 12:43:48 abe Exp $
+ * $Id: dlsof.h,v 1.16 2000/03/14 20:37:26 abe Exp $
  */
 
 
@@ -47,18 +47,39 @@
 #include <unistd.h>
 #include <sys/filedesc.h>
 #include <sys/mbuf.h>
+
+# if	defined(NETBSDV)
+#include <sys/buf.h>
+# endif	/* defined(NETBSDV) */
+
 #define	NFS
 #define m_stat	mnt_stat
 #include <sys/mount.h>
 #include <rpc/types.h>
 #include <sys/protosw.h>
+
+# if	defined(NETBSDV) && NETBSDV>=1030
+#define	sockproto	NETBSD_sockproto
+# endif	/* defined(NETBSDV) && NETBSDV>=1030 */
+
 #include <sys/socket.h>
+
+# if	defined(NETBSDV) && NETBSDV>=1030
+#undef	sockproto
+# endif	/* defined(NETBSDV) && NETBSDV>=1030 */
+
 #include <sys/socketvar.h>
 #include <sys/un.h>
 #include <sys/unpcb.h>
 #include <netinet/in.h>
 #include <netinet/in_systm.h>
 #include <netinet/ip.h>
+
+# if	defined(HASIPv6) && defined(NETBSDV) && !defined(HASINRIAIPv6)
+#include <netinet/ip6.h>
+#include <netinet6/in6_pcb.h>
+# endif	/* defined(HASIPv6) && defined(NETBSDV) && !defined(HASINRIAIPv6) */
+
 #include <net/route.h>
 #include <netinet/in_pcb.h>
 #include <netinet/ip_var.h>
@@ -68,7 +89,35 @@
 #include <netinet/tcp_timer.h>
 #include <netinet/tcp_var.h>
 #include <sys/ucred.h>
+
+# if	defined(UVM)
+/*
+ * Avoid conflicts with definitions in <vm/vm_param.h>.
+ */
+
+#undef	FALSE
+#undef	TRUE
+# endif	/* defined(UVM) */
+
 #include <sys/vnode.h>
+
+# if	defined(NETBSDV) && NETBSDV>=1030
+/*
+ * Because late in the 1.3I NetBSD development cycle the sockproto structure
+ * was placed under _KERNEL in <sys/socket.h>, and because defining _KERNEL
+ * before #include'ing <sys/socket.h> causes other #include problems, the
+ * sockproto structure definition that might have been in <sys/socket.h> is
+ * renamed NETBSD_sockproto, and the following definition is used instead.
+ *
+ * Ugly, isn't it?
+ */
+
+struct sockproto {
+	u_short sp_family;
+	u_short sp_protocol;
+};
+# endif	/* defined(NETBSDV) && NETBSDV>=1030 */
+
 #include <net/raw_cb.h>
 #include <sys/domain.h>
 #define	pmap	RPC_pmap
@@ -103,8 +152,6 @@
 #include <nfs/nfsnode.h>
 #include <sys/proc.h>
 #include <kvm.h>
-#undef	TRUE
-#undef	FALSE
 #include <sys/sysctl.h>
 #define	P_ADDR		kp_eproc.e_paddr
 #define	P_COMM		kp_proc.p_comm
@@ -139,18 +186,38 @@
 #include <sys/lockf.h>
 #undef	KERNEL
 #undef	_KERNEL
+
+# if	defined(UVM)
+#  if	defined(OPENBSDV)
+#define	_UVM_UVM_FAULT_I_H_	1		/* avoid OpenBSD's
+						/* <uvm/uvm_fault_i.h */
+#  endif	/* defined(OPENBSDV) */
+#define	FALSE	0
+#define	TRUE	1
+#include <uvm/uvm.h>
+# endif	/* defined(UVM) */
+
 #include <vm/vm.h>
 #include <vm/vm_map.h>
 #include <vm/vm_object.h>
 #include <vm/vm_pager.h>
 
 # if	defined(OPENBSDV)
+#  if	OPENBSDV==2030 && defined(__sparc__)
+#   if	defined(nbpg)
+#undef	nbpg
+#   endif	/* defined(nbpg) */
+#define	nbpg	4096		/* WARNING!!!  This should be 8192 for sun4,
+				 * but there's not much chance this value will
+				 * ever be used by any lsof code.  (See the
+				 * use of PIPE_NODIRECT in <sys/pipe.h>. */
+#  endif	/* OPENBSDV==2030 && defined(__sparc__) */
 #include <sys/pipe.h>
 #endif	/* defined(OPENBSDV) */
 
 #define	COMP_P		const void
 #define DEVINCR		1024	/* device table malloc() increment */
-#define	KA_T		u_long
+typedef	u_long		KA_T;
 #define	KMEM		"/dev/kmem"
 #define MALLOC_P	void
 #define FREE_P		MALLOC_P
@@ -173,7 +240,7 @@ extern kvm_t *Kd;
 extern KA_T Kpa;
 
 struct l_vfs {
-	struct mount *addr;		/* kernel address */
+	KA_T addr;			/* kernel address */
 	fsid_t	fsid;			/* file system ID */
 	char type[MFSNAMELEN];		/* type of file system */
 	char *dir;			/* mounted directory */
@@ -246,7 +313,7 @@ struct sfile {
 
 #  if	(defined(OPENBSDV) && OPENBSDV>=2010) || (defined(NETBSDV) && NETBSDV>=1020)
 #define	NCACHE_NXT	nc_hash.le_next	/* link in NCACHE */
-  #else	/* (defined(OPENBSDV) && OPENBSDV>=2010) || (defined(NETBSDV) && NETBSDV>=1020) */
+#  else	/* (defined(OPENBSDV) && OPENBSDV>=2010) || (defined(NETBSDV) && NETBSDV>=1020) */
 #   if	defined(NetBSD1_0) && NetBSD<1994101
 #define	NCACHE_NXT	nc_nxt		/* link in NCACHE */
 #   else	/* !defined(NetBSD1_0) || NetBSD>=1994101 */
