@@ -37,7 +37,7 @@
 # if	!defined(lint)
 static char copyright[] =
 "@(#) Copyright 1997 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: dvch.c,v 1.3 97/07/09 08:30:10 abe Exp $";
+static char *rcsid = "$Id: dvch.c,v 1.9 1999/10/23 14:39:54 abe Exp $";
 # endif	/* !defined(lint) */
 
 #include "../lsof.h"
@@ -47,13 +47,19 @@ static char *rcsid = "$Id: dvch.c,v 1.3 97/07/09 08:30:10 abe Exp $";
  *
  * The caller may define the following:
  *
- *	DCACHE_CLONE	is the name of the function that reads and writes
- *			the clone section of the device cache file.  The
- *			clone section follows the device section.
+ *	DCACHE_CLONE	is the name of the function that reads and writes the
+ *			clone section of the device cache file.  The clone
+ *			section follows the device section.  If DCACHE_CLONE
+ *			isn't defined, but HAS_STD_CLONE is defined to be 1,
+ *			DCACHE_CLONE defaults to the local static function
+ *			rw_clone_sect() that reads and writes a standard
+ *			clone cache.
  *
- *	DCACHE_CLR	is the name of the function that clears the clone
- *			and pseudo caches when reading the device cache
- *			fails.
+ *	DCACHE_CLR	is the name of the function that clears the clone and
+ *			pseudo caches when reading the device cache fails.   If
+ *			DCACHE_CLR isn't defined, but HAS_STD_CLONE is defined
+ *			to be 1, DCACHE_CLR defaults to the local static
+ *			function clr_sect() that clears a standard clone cache.
  *
  *	DCACHE_PSEUDO	is the name of the function that reads and writes
  *			the pseudo section of the device cache file.  The
@@ -87,6 +93,29 @@ static char *rcsid = "$Id: dvch.c,v 1.3 97/07/09 08:30:10 abe Exp $";
 static int crctbl[CRC_TBLL];		/* crc partial results table */
 
 
+/*
+ * Local function prototypes
+ */
+
+#undef	DCACHE_CLR_LOCAL
+# if	!defined(DCACHE_CLR)
+#  if	defined(HAS_STD_CLONE) && HAS_STD_CLONE==1
+#define	DCACHE_CLR		clr_sect
+#define	DCACHE_CLR_LOCAL	1
+_PROTOTYPE(static void clr_sect,(void));
+#  endif	/* defined(HAS_STD_CLONE) && HAS_STD_CLONE==1 */
+# endif	/* !defined(DCACHE_CLR) */
+
+#undef	DCACHE_CLONE_LOCAL
+# if	!defined(DCACHE_CLONE)
+#  if	defined(HAS_STD_CLONE) && HAS_STD_CLONE==1
+#define	DCACHE_CLONE		rw_clone_sect
+#define	DCACHE_CLONE_LOCAL	1
+_PROTOTYPE(static int rw_clone_sect,(int m));
+#  endif	/* defined(HAS_STD_CLONE) && HAS_STD_CLONE==1 */
+# endif	/*!defined(DCACHE_CLONE) */
+
+
 # if	defined(HASBLKDEV)
 /*
  * alloc_bdcache() - allocate block device cache
@@ -95,15 +124,15 @@ static int crctbl[CRC_TBLL];		/* crc partial results table */
 void
 alloc_bdcache()
 {
-	if ((BDevtp = (struct l_dev *)calloc((MALLOC_S)BNdev,
-				     sizeof(struct l_dev)))
-	== NULL) {
+	if (!(BDevtp = (struct l_dev *)calloc((MALLOC_S)BNdev,
+				       sizeof(struct l_dev))))
+	{
 	    (void) fprintf(stderr, "%s: no space for block devices\n", Pn);
 	    Exit(1);
 	}
-	if ((BSdev = (struct l_dev **)malloc((MALLOC_S)(sizeof(struct l_dev *)
-	    * BNdev)))
-	== NULL) {
+	if (!(BSdev = (struct l_dev **)malloc((MALLOC_S)(sizeof(struct l_dev *)
+				       * BNdev))))
+	{
 	    (void) fprintf(stderr, "%s: no space for block device pointers\n",
 		Pn);
 	    Exit(1);
@@ -119,15 +148,15 @@ alloc_bdcache()
 void
 alloc_dcache()
 {
-	if ((Devtp = (struct l_dev *)calloc((MALLOC_S)Ndev,
-				     sizeof(struct l_dev)))
-	== NULL) {
+	if (!(Devtp = (struct l_dev *)calloc((MALLOC_S)Ndev,
+				      sizeof(struct l_dev))))
+	{
 	    (void) fprintf(stderr, "%s: no space for devices\n", Pn);
 	    Exit(1);
 	}
-	if ((Sdev = (struct l_dev **)malloc((MALLOC_S)(sizeof(struct l_dev *)
-	    * Ndev)))
-	== NULL) {
+	if (!(Sdev = (struct l_dev **)malloc((MALLOC_S)(sizeof(struct l_dev *)
+				      * Ndev))))
+	{
 	    (void) fprintf(stderr, "%s: no space for device pointers\n",
 		Pn);
 	    Exit(1);
@@ -148,15 +177,15 @@ clr_devtab()
 	    for (i = 0; i < Ndev; i++) {
 		if (Devtp[i].name) {
 		    (void) free((FREE_P *)Devtp[i].name);
-		    Devtp[i].name = NULL;
+		    Devtp[i].name = (char *)NULL;
 		}
 	    }
 	    (void) free((FREE_P *)Devtp);
-	    Devtp = NULL;
+	    Devtp = (struct l_dev *)NULL;
 	}
 	if (Sdev) {
 	    (void) free((FREE_P *)Sdev);
-	    Sdev = NULL;
+	    Sdev = (struct l_dev **)NULL;
 	}
 	Ndev = 0;
 
@@ -165,15 +194,15 @@ clr_devtab()
 	    for (i = 0; i < BNdev; i++) {
 		if (BDevtp[i].name) {
 		    (void) free((FREE_P *)BDevtp[i].name);
-		    BDevtp[i].name = NULL;
+		    BDevtp[i].name = (char *)NULL;
 		}
 	    }
 	    (void) free((FREE_P *)BDevtp);
-	    BDevtp = NULL;
+	    BDevtp = (struct l_dev *)NULL;
 	}
 	if (BSdev) {
 	    (void) free((FREE_P *)BSdev);
-	    BSdev = NULL;
+	    BSdev = (struct l_dev **)NULL;
 	}
 	BNdev = 0;
 # endif	/* defined(HASBLKDEV) */
@@ -181,6 +210,27 @@ clr_devtab()
 }
 
 
+# if	defined(DCACHE_CLR_LOCAL)
+/*
+ * clr_sect() - clear cached standard clone sections
+ */
+
+static void
+clr_sect()
+{
+	struct clone *c, *c1;
+
+	if (Clone) {
+	    for (c = Clone; c; c = c1) {
+		c1 = c->next;
+		(void) free((FREE_P *)c);
+	    }
+	    Clone = (struct clone *)NULL;
+	}
+}
+# endif	/* defined(DCACHE_CLR_LOCAL) */
+
+
 /*
  * crc(b, l, s) - compute a crc for a block of bytes
  */
@@ -243,11 +293,11 @@ dcpath(rw, npw)
 {
 	char buf[MAXPATHLEN+1], *cp1, *cp2, hn[MAXPATHLEN+1];
 	int endf;
-	int i, j, l;
+	int i, j;
+	int l = 0;
 	int ierr = 0;			/* intermediate error state */
 	int merr = 0;			/* malloc error state */
-	struct passwd *p = NULL;
-	uid_t ul;
+	struct passwd *p = (struct passwd *)NULL;
 	static short wenv = 1;		/* HASENVDC warning state */
 	static short wpp = 1;		/* HASPERSDCPATH warning state */
 /*
@@ -255,11 +305,11 @@ dcpath(rw, npw)
  */
 	if (DCpath[1]) {
 	    (void) free((FREE_P *)DCpath[1]);
-	    DCpath[1] = NULL;
+	    DCpath[1] = (char *)NULL;
 	}
 	if (DCpath[3]) {
 	    (void) free((FREE_P *)DCpath[3]);
-	    DCpath[3] = NULL;
+	    DCpath[3] = (char *)NULL;
 	}
 /*
  * If a path was specified via -D, it's character address will have been
@@ -270,7 +320,7 @@ dcpath(rw, npw)
 	if (Myuid == 0 || rw == 1 || (!Setuidroot && !Setgid))
 	    DCpath[0] = DCpathArg;
 	else
-	    DCpath[0] = NULL;
+	    DCpath[0] = (char *)NULL;
 
 # if	defined(HASENVDC)
 /*
@@ -278,22 +328,21 @@ dcpath(rw, npw)
  * is a setuid-root process, or the real UID of the process is 0, or the
  * mode is write and the process is setgid.
  */
-	if ((cp1 = getenv(HASENVDC)) != NULL && (l = strlen(cp1)) > 0
+	if ((cp1 = getenv(HASENVDC)) && (l = strlen(cp1)) > 0
 	&&  !Setuidroot && Myuid && (rw == 1 || !Setgid)) {
-	    if ((cp2 = (char *)malloc((MALLOC_S)(l + 1))) == NULL) {
+	    if (!(cp2 = mkstrcpy(cp1, (MALLOC_S *)NULL))) {
 		(void) fprintf(stderr,
-		    "%s: no space for device cache path: %s=%s\n",
-		    Pn, HASENVDC, cp1);
+		    "%s: no space for device cache path: %s=", Pn, HASENVDC);
+		safestrprt(cp1, stderr, 1);
 		merr = 1;
-	    } else {
-		(void) strcpy(cp2, cp1);
+	    } else
 		DCpath[1] = cp2;
-	    }
 	} else if (cp1 && l > 0) {
-	    if (!Fwarn && wenv)
+	    if (!Fwarn && wenv) {
 		(void) fprintf(stderr,
-		    "%s: WARNING: ignoring environment: %s=%s\n",
-		    Pn, HASENVDC, cp1);
+		    "%s: WARNING: ignoring environment: %s=", Pn, HASENVDC);
+		safestrprt(cp1, stderr, 1);
+	    }
 	    wenv = 0;
 	}
 # endif	/* defined(HASENVDC) */
@@ -306,7 +355,7 @@ dcpath(rw, npw)
 	if (rw != 2)
 	    DCpath[2] = HASSYSDC;
 	else
-	    DCpath[2] = NULL;
+	    DCpath[2] = (char *)NULL;
 # endif	/* defined(HASSYSDC) */
 
 # if	defined(HASPERSDC)
@@ -323,7 +372,7 @@ dcpath(rw, npw)
 	    /*
 	     * If the format character isn't a `%', copy it.
 	     */
-		if (i < sizeof(buf)) {
+		if (i < (int)sizeof(buf)) {
 		    buf[i++] = *cp1;
 		    continue;
 		} else {
@@ -344,7 +393,7 @@ dcpath(rw, npw)
 	     */
 
 	    case '%':
-		if (i < sizeof(buf))
+		if (i < (int)sizeof(buf))
 			buf[i++] = '%';
 		else
 			ierr = 2;
@@ -375,16 +424,15 @@ dcpath(rw, npw)
 	     */
 	
 	    case 'h':
-		if (p == NULL && (p = getpwuid(Myuid)) == NULL) {
-		    if (!Fwarn) {
+		if (!p && !(p = getpwuid(Myuid))) {
+		    if (!Fwarn)
 			(void) fprintf(stderr,
 			    "%s: WARNING: can't get home dir for UID: %d\n",
-			    Pn, Myuid);
-		    }
+			    Pn, (int)Myuid);
 		    ierr = 1;
 		    break;
 		}
-		if ((i + (l = strlen(p->pw_dir))) >= sizeof(buf)) {
+		if ((i + (l = strlen(p->pw_dir))) >= (int)sizeof(buf)) {
 		    ierr = 2;
 		    break;
 		}
@@ -412,19 +460,18 @@ dcpath(rw, npw)
 	    case 'l':
 	    case 'L':
 		if (gethostname(hn, sizeof(hn) - 1) < 0) {
-		    if (!Fwarn) {
+		    if (!Fwarn)
 			(void) fprintf(stderr,
-			    "%s: WARNING: no gethostname for %l or %L: %s\n",
+			    "%s: WARNING: no gethostname for %%l or %%L: %s\n",
 			    Pn, strerror(errno));
-		    }
 		    ierr = 1;
 		    break;
 		}
 		hn[sizeof(hn) - 1] = '\0';
-		if (*cp1 == 'L' && (cp2 = strchr(hn, '.')) != NULL && cp2 > hn)
+		if (*cp1 == 'L' && (cp2 = strchr(hn, '.')) && cp2 > hn)
 		    *cp2 = '\0';
 		j = strlen(hn);
-		if ((j + i) < sizeof(buf)) {
+		if ((j + i) < (int)sizeof(buf)) {
 		    (void) strcpy(&buf[i], hn);
 		    i += j;
 		} else
@@ -445,7 +492,7 @@ dcpath(rw, npw)
 	    case 'p':
 
 #  if	defined(HASPERSDCPATH)
-		if ((cp2 = getenv(HASPERSDCPATH)) != NULL
+		if ((cp2 = getenv(HASPERSDCPATH))
 		&&  (l = strlen(cp2)) > 0
 		&&  !Setuidroot
 		&&  Myuid
@@ -455,11 +502,11 @@ dcpath(rw, npw)
 			cp2++;
 			l--;
 		    }
-		    if ((i + l) < (sizeof(buf) - 1)) {
+		    if ((i + l) < ((int)sizeof(buf) - 1)) {
 			(void) strcpy(&buf[i], cp2);
 			i += l;
 			if (buf[i - 1] != '/') {
-			    if (i < sizeof(buf) - 2) {
+			    if (i < ((int)sizeof(buf) - 2)) {
 				buf[i++] = '/';
 				buf[i] = '\0';
 			    } else
@@ -471,18 +518,18 @@ dcpath(rw, npw)
 		    if (cp2 && l > 0)  {
 			if (!Fwarn && wpp) {
 			    (void) fprintf(stderr,
-				"%s: WARNING: ignoring environment: %s=%s\n",
-				Pn, HASPERSDCPATH, cp2);
+				"%s: WARNING: ignoring environment: %s",
+				Pn, HASPERSDCPATH);
+			    safestrprt(cp2, stderr, 1);
 			} 
 			wpp = 0;
 		    }
 		}
 #  else	/* !defined(HASPERSDCPATH) */
-		if (!Fwarn && wpp) {
+		if (!Fwarn && wpp)
 		    (void) fprintf(stderr,
 			"%s: WARNING: HASPERSDCPATH disabled: %s\n",
 			Pn, HASPERSDC);
-		}
 		ierr = 1;
 		wpp = 0;
 #  endif	/* defined(HASPERSDCPATH) */
@@ -495,16 +542,15 @@ dcpath(rw, npw)
 	     */
 
 	    case 'u':
-		if (p == NULL && (p = getpwuid(Myuid)) == NULL) {
-		    if (!Fwarn) {
+		if (!p && !(p = getpwuid(Myuid))) {
+		    if (!Fwarn)
 			(void) fprintf(stderr,
 			    "%s: WARNING: can't get login name for UID: %d\n",
-			    Pn, Myuid);
-		    }
+			    Pn, (int)Myuid);
 		    ierr = 1;
 		    break;
 		}
-		if ((i + (l = strlen(p->pw_name))) >= sizeof(buf)) {
+		if ((i + (l = strlen(p->pw_name))) >= (int)sizeof(buf)) {
 		    ierr = 2;
 		    break;
 		}
@@ -517,8 +563,8 @@ dcpath(rw, npw)
 	     */
 
 	    case 'U':
-		(void) sprintf(hn, "%d", Myuid);
-		if ((i + (l = strlen(hn))) >= sizeof(buf))
+		(void) sprintf(hn, "%d", (int)Myuid);
+		if ((i + (l = strlen(hn))) >= (int)sizeof(buf))
 		    ierr = 2;
 		else {
 		    (void) strcpy(&buf[i], hn);
@@ -526,11 +572,10 @@ dcpath(rw, npw)
 		}
 		break;
 	    default:
-		if (!Fwarn) {
+		if (!Fwarn)
 		    (void) fprintf(stderr,
 			"%s: WARNING: bad conversion (%%%c): %s\n",
 			Pn, *cp1, HASPERSDC);
-		}
 		ierr = 1;
 	    }
 	    if (endf || ierr > 1)
@@ -544,11 +589,10 @@ dcpath(rw, npw)
 	 * warning message.  A type 2 intermediate error requires the
 	 * issuing of a buffer overlow warning message.
 	 */
-	    if (ierr == 2 && !Fwarn) {
+	    if (ierr == 2 && !Fwarn)
 		(void) fprintf(stderr,
 	 	    "%s: WARNING: device cache path too large: %s\n",
 		    Pn, HASPERSDC);
-	    }
 	    i = 0;
 	}
 	buf[i] = '\0';
@@ -557,15 +601,13 @@ dcpath(rw, npw)
  * copy buf[] to it, and store its pointer in DCpath[3].
  */
 	if (i) {
-	    if ((cp1 = (char *)malloc((MALLOC_S)(i + 1))) == NULL) {
+	    if (!(cp1 = mkstrcpy(buf, (MALLOC_S *)NULL))) {
 		(void) fprintf(stderr,
-		    "%s: no space for device cache path: %s\n",
-		    Pn, buf);
+		    "%s: no space for device cache path: ", Pn);
+		safestrprt(buf, stderr, 1);
 		merr = 1;
-	    } else {
-		(void) strcpy(cp1, buf);
+	    } else
 		DCpath[3] = cp1;
-	    }
 	}
 # endif	/* defined(HASPERSDC) */
 
@@ -584,10 +626,9 @@ dcpath(rw, npw)
 	    if (DCpath[i])
 		return(i);
 	}
-	if (!Fwarn && npw) {
+	if (!Fwarn && npw)
 	    (void) fprintf(stderr,
 		"%s: WARNING: can't form any device cache path\n", Pn);
-	}
 	return(-1);
 }
 
@@ -603,8 +644,7 @@ open_dcache(m, r, s)
 	struct stat *s;		/* stat() receiver */
 {
 	char buf[128];
-	int fp, lp;
-	char *w = NULL;
+	char *w = (char *)NULL;
 /*
  * Get the device cache file paths.
  */
@@ -654,7 +694,7 @@ close_exit:
 		DCfd = -1;
 		return(1);
 	    }
-	    if ((s->st_mode & 07777) != ((DCpathX == 2) ? 0644 : 0600)) {
+	    if ((int)(s->st_mode & 07777) != ((DCpathX == 2) ? 0644 : 0600)) {
 		(void) sprintf(buf, "doesn't have %04o modes",
 	    	    (DCpathX == 2) ? 0644 : 0600);
 		w = buf;
@@ -703,8 +743,6 @@ close_exit:
 			(void) fprintf(stderr,
 			     "%s: WARNING: can't change ownerships of %s: %s\n",
 			     Pn, DCpath[DCpathX], strerror(errno));
-		    (void) unlink(DCpath[DCpathX]);
-		    goto close_exit;
 	 	}
 	    }
 	    if (!Fwarn && DCstate != 1 && !DCunsafe)
@@ -725,7 +763,7 @@ close_exit:
 		Pn, m);
 	    Exit(1);
 	}
-	return(1);		/* to make IRIX 6.0's C compiler happy */
+	return(1);
 }
 
 
@@ -748,7 +786,7 @@ read_dcache()
  */
 	if ((ov = open_dcache(1, 0, &sb)) != 0) {
 	    if (DCpathX == 2) {
-		if (ov == 2 && DCpath[3] != NULL) {
+		if (ov == 2 && DCpath[3]) {
 		    DCpathX = 3;
 		    if (open_dcache(1, 1, &sb) != 0)
 			return(1);
@@ -770,7 +808,7 @@ read_dcache()
 	    if (sb.st_mtime <= devsb.st_mtime || sb.st_ctime <= devsb.st_ctime)
 		DCunsafe = 1;
 	}
-	if ((DCfs = fdopen(DCfd, "r")) == NULL) {
+	if (!(DCfs = fdopen(DCfd, "r"))) {
 	    if (!Fwarn)
 		(void) fprintf(stderr,
 		    "%s: WARNING: can't fdopen(%s)\n", Pn, DCpath[DCpathX]);
@@ -782,7 +820,7 @@ read_dcache()
  * Read the section count line; initialize the CRC table;
  * validate the section count line.
  */
-	if (fgets(buf, sizeof(buf), DCfs) == NULL) {
+	if (!fgets(buf, sizeof(buf), DCfs)) {
 
 cant_read:
 	    if (!Fwarn)
@@ -792,7 +830,7 @@ cant_read:
 read_close:
 		(void) fclose(DCfs);
 		DCfd = -1;
-		DCfs = NULL;
+		DCfs = (FILE *)NULL;
 		(void) clr_devtab();
 
 # if	defined(DCACHE_CLR)
@@ -824,26 +862,30 @@ read_close:
 
 	(void) sprintf(cbuf, "%d section%s\n", i, cp);
 	if (strcmp(buf, cbuf) != 0) {
-	    if (!Fwarn)
+	    if (!Fwarn) {
 		(void) fprintf(stderr,
-		    "%s: WARNING: bad section count line in %s: %s",
-		    Pn, DCpath[DCpathX], buf);
+		    "%s: WARNING: bad section count line in %s: ",
+		    Pn, DCpath[DCpathX]);
+		safestrprt(buf, stderr, 1);
+	    }
 	    goto read_close;
 	}
 /*
  * Read device section header and validate it.
  */
-	if (fgets(buf, sizeof(buf), DCfs) == NULL)
+	if (!fgets(buf, sizeof(buf), DCfs))
 	    goto cant_read;
 	(void) crc(buf, strlen(buf), &DCcksum);
 	len = strlen("device section: ");
 	if (strncmp(buf, "device section: ", len) != 0) {
 
 read_dhdr:
-	    if (!Fwarn)
+	    if (!Fwarn) {
 		(void) fprintf(stderr,
-		    "%s: WARNING: bad device section header in %s: %s\n",
-		    Pn, DCpath[DCpathX], buf);
+		    "%s: WARNING: bad device section header in %s: ",
+		    Pn, DCpath[DCpathX]);
+		safestrprt(buf, stderr, 1);
+	    }
 	    goto read_close;
 	}
 /*
@@ -857,7 +899,7 @@ read_dhdr:
  * Construct the Sdev[] pointers to Devtp[].
  */
 	for (i = 0; i < Ndev; i++) {
-	    if (fgets(buf, sizeof(buf), DCfs) == NULL) {
+	    if (!fgets(buf, sizeof(buf), DCfs)) {
 		if (!Fwarn)
 		    (void) fprintf(stderr,
 			"%s: WARNING: can't read device %d from %s\n",
@@ -868,11 +910,13 @@ read_dhdr:
 	/*
 	 * Convert hexadecimal device number.
 	 */
-	    if ((cp = x2dev(buf, &Devtp[i].rdev)) == NULL || *cp != ' ') {
-		if (!Fwarn)
+	    if (!(cp = x2dev(buf, &Devtp[i].rdev)) || *cp != ' ') {
+		if (!Fwarn) {
 		    (void) fprintf(stderr,
-			"%s: device %d: bad device in %s: %s",
-			Pn, i + 1, DCpath[DCpathX], buf);
+			"%s: device %d: bad device in %s: ",
+			Pn, i + 1, DCpath[DCpathX]);
+		    safestrprt(buf, stderr, 1);
+		}
 		goto read_close;
 	    }
 	/*
@@ -880,10 +924,12 @@ read_dhdr:
 	 */
 	    for (cp++, Devtp[i].inode = 0; *cp != ' '; cp++) {
 		if (*cp < '0' || *cp > '9') {
-		    if (!Fwarn)
+		    if (!Fwarn) {
 			(void) fprintf(stderr,
-			    "%s: WARNING: device %d: bad inode # in %s: %s",
-			    Pn, i + 1, DCpath[DCpathX], buf);
+			    "%s: WARNING: device %d: bad inode # in %s: ",
+			    Pn, i + 1, DCpath[DCpathX]);
+			safestrprt(buf, stderr, 1);
+		    }
 		    goto read_close;
 		}
 		Devtp[i].inode = (Devtp[i].inode * 10) + (int)(*cp - '0');
@@ -894,19 +940,20 @@ read_dhdr:
 	 * pointer to Devtp[].
 	 */
 	    if ((len = strlen(++cp)) < 2 || *(cp + len - 1) != '\n') {
-		if (!Fwarn)
+		if (!Fwarn) {
 		    (void) fprintf(stderr,
-			"%s: WARNING: device %d: bad path in %s: %s",
-			Pn, i + 1, DCpath[DCpathX], buf);
-		    goto read_close;
-	    }
-	    if ((Devtp[i].name = (char *)malloc(len)) == NULL) {
-		(void) fprintf(stderr,
-		    "%s: no space for %s", Pn, buf);
-		Exit(1);
+			"%s: WARNING: device %d: bad path in %s: ",
+			Pn, i + 1, DCpath[DCpathX]);
+		    safestrprt(cp, stderr, 1);
+		 }
+		 goto read_close;
 	    }
 	    *(cp + len - 1) = '\0';
-	    (void) strcpy(Devtp[i].name, cp);
+	    if (!(Devtp[i].name = mkstrcpy(cp, (MALLOC_S *)NULL))) {
+		(void) fprintf(stderr, "%s: no space for ", Pn);
+		safestrprt(buf, stderr, 1);
+		Exit(1);
+	    }
 	    Devtp[i].v = 0;
 	    Sdev[i] = &Devtp[i];
 	}
@@ -915,17 +962,17 @@ read_dhdr:
 /*
  * Read block device section header and validate it.
  */
-	if (fgets(buf, sizeof(buf), DCfs) == NULL)
+	if (!fgets(buf, sizeof(buf), DCfs))
 	    goto cant_read;
 	(void) crc(buf, strlen(buf), &DCcksum);
 	len = strlen("block device section: ");
 	if (strncmp(buf, "block device section: ", len) != 0) {
-
-read_bdhdr:
-	    if (!Fwarn)
+	    if (!Fwarn) {
 		(void) fprintf(stderr,
-		    "%s: WARNING: bad block device section header in %s: %s\n",
-		    Pn, DCpath[DCpathX], buf);
+		    "%s: WARNING: bad block device section header in %s: ",
+		    Pn, DCpath[DCpathX]);
+		safestrprt(buf, stderr, 1);
+	    }
 	    goto read_close;
 	}
 /*
@@ -938,7 +985,7 @@ read_bdhdr:
 	 * Construct the BSdev[] pointers to BDevtp[].
 	 */
 	    for (i = 0; i < BNdev; i++) {
-		if (fgets(buf, sizeof(buf), DCfs) == NULL) {
+		if (!fgets(buf, sizeof(buf), DCfs)) {
 		    if (!Fwarn)
 			(void) fprintf(stderr,
 			    "%s: WARNING: can't read block device %d from %s\n",
@@ -949,11 +996,13 @@ read_bdhdr:
 	    /*
 	     * Convert hexadecimal device number.
 	     */
-		if ((cp = x2dev(buf, &BDevtp[i].rdev)) == NULL || *cp != ' ') {
-		    if (!Fwarn)
+		if (!(cp = x2dev(buf, &BDevtp[i].rdev)) || *cp != ' ') {
+		    if (!Fwarn) {
 			(void) fprintf(stderr,
-			    "%s: device %d: bad block device in %s: %s",
-			    Pn, i + 1, DCpath[DCpathX], buf);
+			    "%s: device %d: bad block device in %s: ",
+			    Pn, i + 1, DCpath[DCpathX]);
+			safestrprt(buf, stderr, 1);
+		    }
 		    goto read_close;
 		}
 	    /*
@@ -961,11 +1010,13 @@ read_bdhdr:
 	     */
 		for (cp++, BDevtp[i].inode = 0; *cp != ' '; cp++) {
 		    if (*cp < '0' || *cp > '9') {
-		      if (!Fwarn)
+		      if (!Fwarn) {
 			(void) fprintf(stderr,
-			  "%s: WARNING: block device %d: bad inode # in %s: %s",
-			  Pn, i + 1, DCpath[DCpathX], buf);
-		        goto read_close;
+			  "%s: WARNING: block device %d: bad inode # in %s: ",
+			  Pn, i + 1, DCpath[DCpathX]);
+			safestrprt(buf, stderr, 1);
+		      }
+		      goto read_close;
 		    }
 		    BDevtp[i].inode = (BDevtp[i].inode * 10) + (int)(*cp - '0');
 		}
@@ -975,18 +1026,20 @@ read_bdhdr:
 	     * pointer to BDevtp[].
 	     */
 		if ((len = strlen(++cp)) < 2 || *(cp + len - 1) != '\n') {
-		    if (!Fwarn)
+		    if (!Fwarn) {
 			(void) fprintf(stderr,
-			    "%s: WARNING: block device %d: bad path in %s: %s",
-			    Pn, i + 1, DCpath[DCpathX], buf);
+			    "%s: WARNING: block device %d: bad path in %s: ",
+			    Pn, i + 1, DCpath[DCpathX]);
+			safestrprt(buf, stderr, 1);
+		    }
 		    goto read_close;
 		}
-		if ((BDevtp[i].name = (char *)malloc(len)) == NULL) {
-		    (void) fprintf(stderr, "%s: no space for %s", Pn, buf);
+	        *(cp + len - 1) = '\0';
+		if (!(BDevtp[i].name = mkstrcpy(cp, (MALLOC_S *)NULL))) {
+		    (void) fprintf(stderr, "%s: no space for ", Pn);
+		    safestrprt(cp, stderr, 1);
 		    Exit(1);
 		}
-	        *(cp + len - 1) = '\0';
-	        (void) strcpy(BDevtp[i].name, cp);
 	        BDevtp[i].v = 0;
 	        BSdev[i] = &BDevtp[i];
 	    }
@@ -1013,25 +1066,29 @@ read_bdhdr:
  * Read and check the CRC section; it must be the last thing in the file.
  */
 	(void) sprintf(cbuf, "CRC section: %x\n", DCcksum);
-	if (fgets(buf, sizeof(buf), DCfs) == NULL || strcmp(buf, cbuf) != 0) {
-	    if (!Fwarn)
+	if (!fgets(buf, sizeof(buf), DCfs) || strcmp(buf, cbuf) != 0) {
+	    if (!Fwarn) {
 		(void) fprintf(stderr,
-		    "%s: WARNING: bad CRC section in %s: %s",
-		    Pn, DCpath[DCpathX], buf);
+		    "%s: WARNING: bad CRC section in %s: ",
+		    Pn, DCpath[DCpathX]);
+		safestrprt(buf, stderr, 1);
+	    }
 	    goto read_close;
 	}
-	if (fgets(buf, sizeof(buf), DCfs) != NULL) {
-	    if (!Fwarn)
+	if (fgets(buf, sizeof(buf), DCfs)) {
+	    if (!Fwarn) {
 		(void) fprintf(stderr,
-		    "%s: WARNING: data follows CRC section in %s: %s",
-		    Pn, DCpath[DCpathX], buf);
+		    "%s: WARNING: data follows CRC section in %s: ",
+		    Pn, DCpath[DCpathX]);
+		safestrprt(buf, stderr, 1);
+	    }
 	    goto read_close;
 	}
 /*
  * Check one device entry at random -- the randomness based on our
  * PID.
  */
-	i = Mypid % Ndev;
+	i = (int)(Mypid % Ndev);
 	if (stat(Devtp[i].name, &sb) != 0
 
 # if	defined(DVCH_EXPDEV)
@@ -1052,11 +1109,141 @@ read_bdhdr:
  */
 	(void) fclose(DCfs);
 	DCfd = -1;
-	DCfs = NULL;
+	DCfs = (FILE *)NULL;
 	return(0);
 }
 
 
+# if	defined(DCACHE_CLONE_LOCAL)
+/*
+ * rw_clone_sect() - read/write the device cache file clone section
+ */
+
+static int
+rw_clone_sect(m)
+	int m;				/* mode: 1 = read; 2 = write */
+{
+	char buf[MAXPATHLEN*2], *cp, *cp1;
+	struct clone *c;
+	struct l_dev *dp;
+	int i, j, len, n;
+
+	if (m == 1) {
+
+	/*
+	 * Read the clone section header and validate it.
+	 */
+	    if (!fgets(buf, sizeof(buf), DCfs)) {
+
+bad_clone_sect:
+		if (!Fwarn) {
+		    (void) fprintf(stderr,
+			"%s: bad clone section header in %s: ",
+			Pn, DCpath[DCpathX]);
+		    safestrprt(buf, stderr, 1);
+		}
+		return(1);
+	    }
+	    (void) crc(buf, strlen(buf), &DCcksum);
+	    len = strlen("clone section: ");
+	    if (strncmp(buf, "clone section: ", len) != 0)
+		goto bad_clone_sect;
+	    if ((n = atoi(&buf[len])) < 0)
+		goto bad_clone_sect;
+	/*
+	 * Read the clone section lines and create the Clone list.
+	 */
+	    for (i = 0; i < n; i++) {
+		if (fgets(buf, sizeof(buf), DCfs) == NULL) {
+		    if (!Fwarn) {
+			(void) fprintf(stderr,
+			    "%s: bad clone line in %s: ", Pn, DCpath[DCpathX]);
+			safestrprt(buf, stderr, 1);
+		    }
+		    return(1);
+		}
+		(void) crc(buf, strlen(buf), &DCcksum);
+	    /*
+	     * Assemble Devtp[] index and make sure it's correct.
+	     */
+		for (cp = buf, j = 0; *cp != ' '; cp++) {
+		    if (*cp < '0' || *cp > '9') {
+
+bad_clone_index:
+			if (!Fwarn) {
+			    (void) fprintf(stderr,
+				"%s: bad cached clone device index: ", Pn);
+			    safestrprt(buf, stderr, 1);
+			}
+			return(1);
+		    }
+		    j = (j * 10) + (int)(*cp - '0');
+		}
+		if (j < 0 || j >= Ndev || (cp1 = strchr(++cp, '\n')) == NULL)
+		    goto bad_clone_index;
+		if (strncmp(cp, Devtp[j].name, (cp1 - cp)) != 0)
+		    goto bad_clone_index;
+	    /*
+	     * Allocate and complete a clone structure.
+	     */
+		if (!(c = (struct clone *)malloc(sizeof(struct clone)))) {
+		    (void) fprintf(stderr,
+			"%s: no space for cached clone: ", Pn);
+		    safestrprt(buf, stderr, 1);
+		    Exit(1);
+		}
+		c->dx = j;
+		c->next = Clone;
+		Clone = c;
+	    }
+	    return(0);
+	} else if (m == 2) {
+
+	/*
+	 * Write the clone section header.
+	 */
+	    for (c = Clone, n = 0; c; c = c->next, n++)
+		;
+	    (void) sprintf(buf, "clone section: %d\n", n);
+	    if (wr2DCfd(buf, &DCcksum))
+		return(1);
+	/*
+	 * Write the clone section lines.
+	 */
+	    for (c = Clone; c; c = c->next) {
+		for (dp = &Devtp[c->dx], j = 0; j < Ndev; j++) {
+		    if (dp == Sdev[j])
+			break;
+		}
+		if (j >= Ndev) {
+		    if (!Fwarn) {
+			(void) fprintf(stderr,
+			    "%s: can't make index for clone: ", Pn);
+			safestrprt(dp->name, stderr, 1);
+		    }
+		    (void) unlink(DCpath[DCpathX]);
+		    (void) close(DCfd);
+		    DCfd = -1;
+		    return(1);
+		}
+		(void) sprintf(buf, "%d %s\n", j, dp->name);
+		if (wr2DCfd(buf, &DCcksum))
+		    return(1);
+	    }
+	    return(0);
+	}
+/*
+ * A shouldn't-happen case: mode neither 1 nor 2.
+ */
+	(void) fprintf(stderr, "%s: internal rw_clone_sect error: %d\n",
+	    Pn, m);
+	Exit(1);
+	return(1);		/* This useless return(1) keeps some
+				 * compilers happy. */
+}
+# endif	/* defined(DCACHE_CLONE_LOCAL) */
+
+
 /*
  * write_dcache() - write device cache file
  */
@@ -1104,7 +1291,7 @@ write_dcache()
 	    return;
 	for (i = 0; i < Ndev; i++) {
 	    dp = Sdev[i];
-	    (void) sprintf(buf, "%x %ld %s\n", dp->rdev, (long)dp->inode,
+	    (void) sprintf(buf, "%lx %ld %s\n", (long)dp->rdev, (long)dp->inode,
 		dp->name);
 	    if (wr2DCfd(buf, &DCcksum))
 		return;
@@ -1120,8 +1307,8 @@ write_dcache()
 	if (BNdev) {
 	    for (i = 0; i < BNdev; i++) {
 		dp = BSdev[i];
-		(void) sprintf(buf, "%x %ld %s\n", dp->rdev, (long)dp->inode,
-		    dp->name);
+		(void) sprintf(buf, "%x %ld %s\n", (int)dp->rdev,
+		    (long)dp->inode, dp->name);
 		if (wr2DCfd(buf, &DCcksum))
 		    return;
 	    }
@@ -1148,7 +1335,7 @@ write_dcache()
  * Write the CRC section and close the file.
  */
 	(void) sprintf(buf, "CRC section: %x\n", DCcksum);
-	if (wr2DCfd(buf, NULL))
+	if (wr2DCfd(buf, (unsigned *)NULL))
 		return;
 	if (close(DCfd) != 0) {
 	    if (!Fwarn)
