@@ -37,7 +37,7 @@
 # if	!defined(lint)
 static char copyright[] =
 "@(#) Copyright 1997 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: rnch.c,v 1.5 98/03/06 08:41:03 abe Exp $";
+static char *rcsid = "$Id: rnch.c,v 1.7 98/11/25 06:34:44 abe Exp $";
 # endif	/* !defined(lint) */
 
 #include "../lsof.h"
@@ -265,7 +265,7 @@ ncache_isroot(va, cp)
 		return(0);
 	    if ((len + 1 + strlen(cp) + 1) > sizeof(buf))
 		return(0);
-	    for (mtp = Mtab; mtp; mtp = mtp->next) {
+	    for (mtp = readmnt(); mtp; mtp = mtp->next) {
 		if (!mtp->dir || !mtp->inode)
 		    continue;
 		if (strcmp(Lf->fsdir, mtp->dir) == 0)
@@ -414,7 +414,7 @@ ncache_load()
 	 * Allocate space for the local cache.
 	 */
 	    len = Nc * sizeof(struct l_nch);
-	    if (!(Ncache = (struct l_nch *)malloc((MALLOC_S)len))) {
+	    if (!(Ncache = (struct l_nch *)calloc(Nc, sizeof(struct l_nch)))) {
 
 no_local_space:
 
@@ -728,7 +728,7 @@ ncache_lookup(buf, blen, fp)
 	 */
 	    if (!Lf->fsdir || !Lf->dev_def || Lf->inp_ty != 1)
 		return((char *)NULL);
-	    for (mtp = Mtab; mtp; mtp = mtp->next) {
+	    for (mtp = readmnt(); mtp; mtp = mtp->next) {
 		if (!mtp->dir || !mtp->inode)
 		    continue;
 		if (Lf->dev == mtp->dev
