@@ -31,7 +31,7 @@
 
 
 /*
- * $Id: lsof.h,v 1.12 98/02/16 15:54:28 abe Exp $
+ * $Id: lsof.h,v 1.36 2000/03/16 15:24:23 abe Exp $
  */
 
 
@@ -39,6 +39,11 @@
 #define	LSOF_H	1
 
 #include "machine.h"
+
+# if	!defined(FSV_DEFAULT)
+#define	FSV_DEFAULT	0
+# endif	/* !defined(FSV_DEFAULT) */
+
 #include "lsof_fields.h"
 
 #include <ctype.h>
@@ -48,10 +53,11 @@
 #include <stdio.h>
 
 #include <sys/stat.h>
+#include <sys/wait.h>
 
 
 /*
- * Definitions of structures that may be needed by dlsof.h
+ * Definitions and structures that may be needed by dlsof.h
  */
 
 struct l_dev {
@@ -62,8 +68,113 @@ struct l_dev {
 					 * (when DCUnsafe == 1) */
 };
 
+
+/*
+ * FILE_FLAG column names
+ */
+
+#define	FF_AIO		"AIO"
+#define	FF_APPEND	"AP"
+#define	FF_ASYNC	"ASYN"
+#define	FF_BLKANDSET	"BAS"
+#define	FF_BLKINUSE	"BKIU"
+#define	FF_BLKSEEK	"BSK"
+#define	FF_CLONE	"CLON"
+#define	FF_CLREAD	"CLRD"
+#define	FF_COPYAVOID	"CA"
+#define	FF_CREAT	"CR"
+#define	FF_DATAFLUSH	"DFLU"
+#define	FF_DEFER	"DF"
+#define	FF_DEFERIND	"DFI"
+#define	FF_DELAY	"DLY"
+#define	FF_DIRECT	"DIR"
+#define	FF_DOCLONE	"DOCL"
+#define	FF_DSYNC	"DSYN"
+#define	FF_EXCL		"EXCL"
+#define	FF_EXEC		"EX"
+#define	FF_EXLOCK	"XL"
+#define	FF_FILE_MBLK	"MBLK"
+#define	FF_FSYNC	"FSYN"
+#define	FF_GCFDEFER	"GCDF"
+#define	FF_GCFMARK	"GCMK"
+#define	FF_GENTTY	"GTTY"
+#define	FF_HASLOCK	"LCK"
+#define	FF_HUP		"HUP"
+#define	FF_KERNEL	"KERN"
+#define	FF_KIOCTL	"KIOC"
+#define	FF_LARGEFILE	"LG"
+#define	FF_MARK		"MK"
+#define	FF_MOUNT	"MNT"
+#define	FF_MSYNC	"MSYN"
+#define	FF_NBDRM	"NBDR"
+#define	FF_NBIO		"NBIO"
+#define	FF_NBLOCK	"NB"
+#define	FF_NBUF		"NBF"
+#define	FF_NMFS		"NMFS"
+#define	FF_NDELAY	"ND"
+#define	FF_NET		"NET"
+#define	FF_NOCACHE	"NC"
+#define	FF_NOCTTY	"NTTY"
+#define	FF_NODSYNC	"NDSY"
+#define	FF_NOTOSTOP	"NOTO"
+#define	FF_NSHARE	"NSH"
+#define	FF_OLRMIRROR	"OLRM"
+#define	FF_POSIX_AIO	"PAIO"
+#define	FF_POSIX_PIPE	"PP"
+#define	FF_RAIOSIG	"RAIO"
+#define	FF_RCACH	"RC"
+#define	FF_READ		"R"
+#define	FF_REVOKED	"REV"
+#define	FF_RSHARE	"RSH"
+#define	FF_RSYNC	"RSYN"
+#define	FF_SETBLK	"BL"
+#define	FF_SHLOCK	"SL"
+#define	FF_SOCKET	"SOCK"
+#define	FF_SQTSH1	"SQS1"
+#define	FF_SQTSH2	"SQS2"
+#define	FF_SQTREPAIR	"SQR"
+#define	FF_SQTSH	"SQSH"
+#define	FF_SQTSVM	"SQSV"
+#define	FF_STOPIO	"STPI"
+#define	FF_SYNC		"SYN"
+#define	FF_SYNCRON	"SWR"
+#define	FF_TCP_MDEVONLY	"TCPM"
+#define	FF_TERMIO	"TIO"
+#define	FF_TRUNC	"TR"
+#define	FF_VHANGUP	"VH"
+#define	FF_VTEXT	"VTXT"
+#define	FF_WAKEUP	"WKUP"
+#define	FF_WAITING	"WTG"
+#define	FF_WRITE	"W"
+
+
+/*
+ * Process open file flag names
+ */
+
+#define	POF_ALLOCATED	"ALLC"
+#define	POF_BNRD	"BR"
+#define	POF_BNWR	"BW"
+#define POF_BNHUP	"BHUP"
+#define	POF_CLOEXEC	"CX"
+#define	POF_CLOSING	"CLSG"
+#define	POF_FDLOCK	"LCK"
+#define	POF_INUSE	"USE"
+#define	POF_MAPPED	"MP"
+#define	POF_FSHMAT	"SHMT"
+#define	POF_RSVWT	"RSVW"
+
 #include "dlsof.h"
 
+#include <sys/types.h>			/* just in case -- because utmp.h
+					 * may need it */
+
+# if	defined(EMPTY)
+#undef	EMPTY
+# endif	/* defined(EMPTY) */
+
+#include <utmp.h>
+
 extern int errno;
 extern char *optarg;
 extern int optind;
@@ -81,16 +192,38 @@ extern int optind;
 					 * column */
 #define	CWD		" cwd"		/* current working directory fd name */
 #define	FDLEN		8		/* fd printing array length */
+#define	FSV_FA		0x1		/* file struct addr status */
+#define	FSV_CT		0x2		/* file struct count status */
+#define	FSV_FG		0x4		/* file struct flags */
+#define	FSV_NI		0x8		/* file struct node ID status */
 #define	IDINCR		10		/* PID/PGRP table malloc() increment */
 
 # if	!defined(INADDR_LOOPBACK)
 #define	INADDR_LOOPBACK	(u_long)0x7f000001
 # endif	/* !defined(INADDR_LOOPBACK) */
 
-#define	IPROTOL		6		/* Internet protocol length */
-#define	LOGINML		9		/* login name length */
+#define	IPROTOL		8		/* Internet protocol length */
+
+# if	!defined(KA_T_FMT_X)
+#define	KA_T_FMT_X	"0x%08lx"	/* format for printing kernel
+					 * addresses in 0x... format */
+# endif	/* !defined(KA_T_FMT_X) */
+
+static struct utmp dummy_utmp;		/* to get login name length */
+#define	LOGINML		sizeof(dummy_utmp.ut_name)
+					/* login name length */
 #define	LPROCINCR	128		/* Lproc[] allocation increment */
 #define	LSOF_URL	"ftp://vic.cc.purdue.edu/pub/tools/unix/lsof"
+#define	MIN_AF_ADDR	sizeof(struct in_addr)
+					/* minimum AF_* address length */
+
+# if	defined(HASIPv6)
+#define	MAX_AF_ADDR	sizeof(struct in6_addr)
+					/* maximum AF_* address length */
+# else	/* !defined(HASIPv6) */
+#define	MAX_AF_ADDR	MIN_AF_ADDR	/* maximum AF_* address length */
+# endif	/* defined(HASIPv6) */
+
 #define	MAXDCPATH	4		/* paths in DCpath[] */
 #define	MAXNWAD		100		/* maximum network addresses */
 
@@ -100,38 +233,46 @@ extern int optind;
 
 #define	N_REGLR		0		/* regular file system node */
 #define	N_AFS		1		/* AFS node */
-#define	N_AUTO		2		/* automount node */
-#define	N_BLK		3		/* block device node */
-#define	N_CACHE		4		/* cached file system node */
-#define	N_CDFS		5		/* CD-ROM node */
-#define	N_CHR		6		/* character device node */
-#define	N_COM		7		/* streams common device node */
-#define	N_DOOR		8		/* DOOR node */
-#define	N_FIFO		9		/* FIFO node */
-#define	N_HSFS		10		/* High Sierra node */
-#define	N_KERN		11		/* BSD /kern node */
-#define	N_LOFS		12		/* loopback node */
-#define	N_MPC		13		/* multiplexed device node */
-#define	N_MVFS		14		/* multi-volume file system node (?) */
-#define	N_NFS		15		/* NFS node */
-#define	N_NM		16		/* named file system node */
-#define	N_OBJF		17		/* objfs file system node */
-#define	N_PCFS		18		/* PC file system node */
-#define	N_PIPE		19		/* pipe device node */
-#define	N_PROC		20		/* /proc node */
-#define	N_SOCK		21		/* sock_vnodeops node */
-#define	N_SPEC		22		/* spec_vnodeops node */
-#define	N_STREAM	23		/* stream node */
-#define	N_TMP		24		/* tmpfs node */
-#define	N_UFS		25		/* Unix file system node */
-#define	N_VXFS		26		/* Veritas file system node */
-#define	N_XFS		27		/* XFS node */
+#define	N_AUSX		2		/* Auspex LFS node */
+#define	N_AUTO		3		/* automount node */
+#define	N_BLK		4		/* block device node */
+#define	N_CACHE		5		/* cached file system node */
+#define	N_CDFS		6		/* CD-ROM node */
+#define	N_CHR		7		/* character device node */
+#define	N_COM		8		/* streams common device node */
+#define	N_DOOR		9		/* DOOR node */
+#define	N_FIFO		10		/* FIFO node */
+#define	N_HSFS		11		/* High Sierra node */
+#define	N_KERN		12		/* BSD /kern node */
+#define	N_LOFS		13		/* loopback node */
+#define	N_MNT		14		/* mount file system device node */
+#define	N_MPC		15		/* multiplexed device node */
+#define	N_MVFS		16		/* multi-volume file system node (?) */
+#define	N_NFS		17		/* NFS node */
+#define	N_NM		18		/* named file system node */
+#define	N_OBJF		19		/* objfs file system node */
+#define	N_PCFS		20		/* PC file system node */
+#define	N_PIPE		21		/* pipe device node */
+#define	N_PROC		22		/* /proc node */
+#define	N_SOCK		23		/* sock_vnodeops node */
+#define	N_SPEC		24		/* spec_vnodeops node */
+#define	N_STREAM	25		/* stream node */
+#define	N_TMP		26		/* tmpfs node */
+#define	N_UFS		27		/* Unix file system node */
+#define	N_VXFS		28		/* Veritas file system node */
+#define	N_XFS		29		/* XFS node */
 
 # if	!defined(OFFDECDIG)
 #define	OFFDECDIG	8		/* maximum number of digits in the
 					 * offset decimal form (0t...) */
 # endif	/* !defined(OFFDECDIG) */
 
+# if	!defined(USELOCALREADDIR)
+#define	CloseDir	closedir	/* use standard closedir() */
+#define	OpenDir		opendir		/* use standard opendir() */
+#define	ReadDir		readdir		/* use standard readdir() */
+# endif	/* !defined(USELOCALREADDIR) */
+
 #define	RPTTM		15		/* default repeat seconds */
 #define	RTD		" rtd"		/* root directory fd name */
 #define	TCPTPI_QUEUES	1		/* report TCP/TPI queue lengths */
@@ -153,6 +294,10 @@ extern int optind;
 					 * modifier */
 # endif	/* !defined(SZOFFTYPE) */
 
+# if	!defined(TIMEVAL_LSOF)
+#define	TIMEVAL_LSOF	timeval
+# endif	/* !defined(TIMEVAL_LSOF) */
+
 # if	!defined(XDR_PMAPLIST)
 #define	XDR_PMAPLIST	xdr_pmaplist
 # endif	/* !defined(XDR_PMAPLIST) */
@@ -172,10 +317,12 @@ extern int CmdColW;
 extern int DevColW;
 #define	FDTTL 		"FD"
 extern int FdColW;
-#define INOTTL		"INODE"
-extern int InoColW;
+#define	NLTTL		"NLINK"
+extern int NlColW;
 #define	NMTTL		"NAME"
 extern int NmColW;
+#define NODETTL		"NODE"
+extern int NodeColW;
 #define OFFTTL		"OFFSET"
 #define	PGRPTTL		"PGRP"
 extern int PgrpColW;
@@ -190,6 +337,16 @@ extern int SzOffColW;
 extern int TypeColW;
 #define	USERTTL		"USER"
 extern int UserColW;
+#define	FCTTL		"FCT"
+extern int FcColW;
+#define	FGTTL		"FILE-FLAG"
+extern int FgColW;
+#define	FSTTL		"FILE-ADDR"
+extern int FsColW;
+#define	NITTL		"NODE-ID"
+extern int NiColW;
+extern char *NiTtl;
+
 
 /*
  * Selection flags
@@ -204,33 +361,44 @@ extern int UserColW;
 #define	SELNA		0x004		/* select socket by address (-i@...) */
 #define	SELNET		0x008		/* select Internet socket files (-i) */
 #define	SELNFS		0x010		/* select NFS files (-N) */
-#define	SELNM		0x020		/* select by name */
-#define	SELPGRP		0x040		/* select process group IDs (-g) */
-#define	SELPID		0x080		/* select PIDs (-p) */
-#define	SELUID		0x100		/* select UIDs (-u) */
-#define	SELUNX		0x200		/* select Unix socket (-U) */
+#define	SELNLINK	0x020		/* select based on link count */
+#define	SELNM		0x040		/* select by name */
+#define	SELPGRP		0x080		/* select process group IDs (-g) */
+#define	SELPID		0x100		/* select PIDs (-p) */
+#define	SELUID		0x200		/* select UIDs (-u) */
+#define	SELUNX		0x400		/* select Unix socket (-U) */
 #define	SELALL		(SELCMD|SELFD|SELNA|SELNET|SELNM|SELNFS|SELPID|SELUID|SELUNX)
 
 /*
  * Structure definitions
  */
 
-#if	defined(HAS_AFS)
+# if	defined(HAS_AFS)
 struct afsnode {			/* AFS pseudo-node structure */
 	dev_t dev;
 	unsigned char ino_st;		/* 1 if inode has a value */
+	unsigned char nlink_st;		/* 1 if nlink has a value */
 	unsigned long inode;
 	unsigned long size;
+	long nlink;
 };
-#endif	/* defined(HAS_AFS) */
+# endif	/* defined(HAS_AFS) */
 
-#if	defined(HASNLIST)
+# if	defined(HAS_STD_CLONE)
+struct clone {
+	int dx;			/* index of device entry in Devtp[] */
+	struct clone *next;	/* forward link */
+};
+extern struct clone *Clone;
+# endif	/* defined(HAS_STD_CLONE) */
+
+# if	defined(HASNLIST)
 struct drive_Nl {			/* data to drive build_Nl() */
 	char *nn;			/* nickname for lookups */
 	char *knm;			/* kernel variable for name list */
 };
 extern struct drive_Nl Drive_Nl[];	/* defined in dstore.c */
-#endif	/* defined(HASNLIST) */
+# endif	/* defined(HASNLIST) */
 
 
 /*
@@ -242,6 +410,13 @@ struct int_lst {
 	int f;				/* find state */
 };
 
+# if	defined(HASFSTRUCT)
+struct pff_tab {			/* print file flags table structure */
+	long val;			/* flag value */
+	char *nm;			/* name to print for flag */
+};
+# endif	/* defined(HASFSTRUCT) */
+
 struct seluid {
 	uid_t uid;			/* User ID */
 	char *lnm;			/* specified login name (NULL = none) */
@@ -250,6 +425,11 @@ struct seluid {
 					 * (meaningful only if excl == 0) */
 };
 
+# if	defined(HASBLKDEV)
+extern struct l_dev *BDevtp, **BSdev;
+extern int BNdev;
+# endif	/* defined(HASBLKDEV) */
+
 extern int CkPasswd;
 
 struct str_lst {
@@ -274,12 +454,10 @@ extern int DCunsafe;
 
 extern int DChelp;
 extern struct l_dev *Devtp;
-
-# if	defined(HASBLKDEV)
-extern struct l_dev *BDevtp, **BSdev;
-extern int BNdev;
-# endif	/* defined(HASBLKDEV) */
-
+extern char **Dstk;
+extern int Dstkn;
+extern int Dstkx;
+extern int ErrStat;
 extern int Fand;
 extern int Fblock;
 extern int Ffield;
@@ -293,17 +471,16 @@ extern int Fncache;
 
 extern int Fnet;
 extern int Fnfs;
+extern int Fnlink;
 extern int Foffset;
 extern int Fovhd;
 extern int Fport;
 extern int FportMap;
 extern int Fpgrp;
-
-# if	defined(HASPPID)
 extern int Fppid;
-# endif	/* defined(HASPPID) */
-
 extern int Fsize;
+extern int Fsv;
+extern int FsvFlagX;
 extern int Ftcptpi;
 extern int Fterse;
 extern int Funix;
@@ -315,12 +492,21 @@ extern int Fwarn;
 extern int Fxopt;
 # endif	/* defined(HASXOPT_VALUE) */
 
-extern struct str_lst *Fdl;
+struct fd_lst {
+	char *nm;			/* file descriptor name -- range if
+					 * NULL */
+	int lo;				/* range start (if nm NULL) */
+	int hi;				/* range end (if nm NULL) */
+	struct fd_lst *next;
+};
+extern struct fd_lst *Fdl;
 
 struct fieldsel {
 	char id;			/* field ID character */
 	unsigned char st;		/* field status */
 	char *nm;			/* field name */
+	int *opt;			/* option variable address */
+	int ov;				/* value to OR with option variable */
 };
 extern struct fieldsel FieldSel[];
 
@@ -334,8 +520,10 @@ struct lfile {
 	unsigned char dev_def;		/* device number definition status */
 	unsigned char inp_ty;		/* inode/iproto type
 					 *	0: neither inode nor iproto
-					 *	1: inode contains number
-					 *	2: iproto contains string */
+					 *	1: print inode in decimal
+					 *	2: iproto contains string
+					 *      3: print inode in hex
+					 */
 	unsigned char is_com;		/* common stream status */
 	unsigned char is_nfs;		/* NFS file status */
 	unsigned char is_stream;	/* stream device status */
@@ -344,8 +532,16 @@ struct lfile {
 	unsigned char is_vxfs;		/* VxFS file status */
 # endif	/* defined(HASVXFS) && defined(HASVXFSDNLC) */
 
+	unsigned char lmi_srch;		/* local mount info search status:
+					 * 1 = printname() search required */
+	unsigned char nlink_def;	/* link count definition status */
 	unsigned char off_def;		/* offset definition status */
 	unsigned char sz_def;		/* size definition status */
+
+# if	defined(HASFSTRUCT)
+	unsigned char fsv;		/* file struct value status */
+# endif	/* defined(HASFSTRUCT) */
+
 	char fd[FDLEN];
 	char iproto[IPROTOL];
 	char type[TYPEL];
@@ -356,6 +552,7 @@ struct lfile {
 	SZOFFTYPE sz;
 	dev_t dev;
 	unsigned long inode;
+	long nlink;			/* link count */
 	char *dev_ch;
 	char *fsdir;			/* file system directory */
 	char *fsdev;			/* file system device */
@@ -365,49 +562,66 @@ struct lfile {
 # endif	/* defined HASFSINO) */
 
 	struct linaddr {		/* local Internet address information */
-		int as;			/* address status: 0 == none */
-		int p;			/* port */
-		struct in_addr ia;	/* Internet address */
+	    int af;			/* address family: 0 for none; AF_INET;
+					 * or AF_INET6 */
+	    int p;			/* port */
+	    union {
+		struct in_addr a4;	/* AF_INET Internet address */
+
+# if	defined(HASIPv6)
+		struct in6_addr a6;	/* AF_INET6 Internet address */
+# endif	/* defined(HASIPv6) */
+
+	    } ia;
 	} li[2];			/* li[0]: local
 					 * li[1]: foreign */
 	struct ltstate {		/* local TCP/TPI state */
 		int type;		/* state type: -1 == none
 					 *		0 == TCP
-					 *		1 == TPI */
+					 *		1 == TPI or socket (SS_*) */
 		union {
 			int i;		/* integer state */
 			unsigned int u;	/* unsigned integer state */
 		} state;
 
-#if	defined(HASTCPTPIQ)
+# if	defined(HASTCPTPIQ)
 		unsigned long rq;	/* receive queue length */
 		unsigned long sq;	/* send queue length */
 		unsigned char rqs;	/* rq status: 0 = none */
 		unsigned char sqs;	/* sq status: 0 = none */
-#endif	/* defined(HASTCPTPIQ) */
+# endif	/* defined(HASTCPTPIQ) */
 
-#if	defined(HASTCPTPIW)
+# if	defined(HASTCPTPIW)
 		unsigned char rws;	/* rw status: 0 = none */
 		unsigned char wws;	/* ww status: 0 = none */
 		unsigned long rw;	/* read window size */
 		unsigned long ww;	/* write window size */
-#endif	/* defined(HASTCPTPIW) */
+# endif	/* defined(HASTCPTPIW) */
 
 	} lts;
 	char *nm;
 	char *nma;			/* NAME column addition */
 
-# if	defined(HASNCACHE)
-	unsigned long na;		/* name cache lookup node address */
-#  if	defined(HASNCAPID)
+# if	defined(HASNCACHE) && HASNCACHE<2
+	KA_T na;			/* file structure's node address */
+# endif	/* defined(HASNCACHE) && HASNCACHE<2 */
+
+# if	defined(HASNCACHE) && defined(HASNCAPID)
 	unsigned long id;		/* capability ID */
-#  endif	/* defined(HASNCAPID) */
-# endif	/* defined(HASNCACHE) */
+# endif	/* defined(HASNCACHE) && defined(HASNCAPID) */
 
 # if	defined(HASLFILEADD)
 	HASLFILEADD
 # endif	/* defined(HASLFILEADD) */
 
+# if	defined(HASFSTRUCT)
+	KA_T fsa;			/* file structure address */
+	long fct;			/* file structure's f_count */
+	long ffg;			/* file structure's f_flag */
+	long pof;			/* process open-file flags */
+	KA_T fna;			/* file structure node address */
+# endif	/* defined(HASFSTRUCT) */
+
 	struct lfile *next;
 };
 extern struct lfile *Lf, *Plf;
@@ -428,7 +642,6 @@ struct lproc {
 extern struct lproc *Lp, *Lproc;
 
 extern char *Memory;
-extern struct mounts *Mtab;
 
 # if	defined(HASPROCFS)
 extern struct mounts *Mtprocfs;
@@ -450,7 +663,7 @@ extern int Ndev;
 extern struct NLIST_TYPE *Nl;
 extern int Nll;
 # endif	/* defined(HASNLIST) */
-
+extern long Nlink;
 extern int Nlproc;
 extern char *Nmlst;
 extern int Npgrp;
@@ -462,19 +675,26 @@ extern int Nuidexcl;
 extern int Nuidincl;
 
 struct nwad {
-	char *arg;				/* argument */
-	char *proto;				/* protocol */
-	unsigned char a[4];			/* address */
-	int sport;				/* starting port */
-	int eport;				/* ending port */
-	int f;					/* find state */
-	struct nwad *next;			/* forward link */
+	char *arg;			/* argument */
+	char *proto;			/* protocol */
+	int af;				/* address family -- e.g.,
+					 * AF_INET, AF_INET6 */
+	unsigned char a[MAX_AF_ADDR];	/* address */
+	int sport;			/* starting port */
+	int eport;			/* ending port */
+	int f;				/* find state */
+	struct nwad *next;		/* forward link */
 };
 extern struct nwad *Nwad;
 
 extern int OffDecDig;
 extern char *Pn;
 
+# if	defined(HASFSTRUCT)
+extern struct pff_tab Pff_tab[];	/* file flags table */
+extern struct pff_tab Pof_tab[];	/* process open file flags table */
+# endif	/* defined(HASFSTRUCT) */
+
 # if	defined(HASPROCFS)
 struct procfsid {
 	pid_t pid;			/* search PID */
