@@ -32,22 +32,18 @@
 #ifndef lint
 static char copyright[] =
 "@(#) Copyright 1994 Purdue Research Foundation.\nAll rights reserved.\n";
-static char *rcsid = "$Id: print.c,v 1.15 98/03/06 07:27:47 abe Exp $";
+static char *rcsid = "$Id: print.c,v 1.35 2000/03/16 15:28:06 abe Exp $";
 #endif
 
 
 #include "lsof.h"
 
-#if	defined(EMPTY)
-#undef	EMPTY
-#endif	/* defined(EMPTY) */
 
-#include <utmp.h>
-
-struct utmp dutmp;			/* dummy entry to get login name size */
+/*
+ * Local definitions, structures and function prototypes
+ */
 
 #define HCINC		64		/* host cache size increase chunk */
-#define LNML sizeof(dutmp.ut_name)
 #define PORTHASHBUCKETS	128		/* port hash bucket count
 					 * !!MUST BE A POWER OF 2!! */
 #define	PORTTABTHRESH	10		/* threshold at which we will switch
@@ -56,8 +52,10 @@ struct utmp dutmp;			/* dummy entry to get login name size */
 					 * and fill_porttab() */
 
 struct hostcache {
-	unsigned char addr[4];
-	char *name;
+	unsigned char a[MAX_AF_ADDR];	/* numeric address */
+	int af;				/* address family -- e.g., AF_INET
+					 * or AF_INET6 */
+	char *name;			/* name */
 };
 
 struct porttab {
@@ -150,28 +148,42 @@ fill_portmap()
 	char buf[128], *cp, *nm;
 	CLIENT *c;
 	int h, port, pr;
-	struct hostent *he;
-	struct sockaddr_in ia;
 	MALLOC_S nl;
 	struct pmaplist *p = (struct pmaplist *)NULL;
 	struct porttab *pt;
 	struct rpcent *r;
+	struct TIMEVAL_LSOF tm;
+
+#if	!defined(CAN_USE_CLNT_CREATE)
+	struct hostent *he;
+	struct sockaddr_in ia;
 	int s = RPC_ANYSOCK;
-	struct timeval tm;
+#endif	/* !defined(CAN_USE_CLNT_CREATE) */
+
 /*
- * Construct a TCP socket address for communicating with the portmapper.
+ * Construct structures for communicating with the portmapper.
  */
+
+#if	!defined(CAN_USE_CLNT_CREATE)
 	zeromem(&ia, sizeof(ia));
 	ia.sin_family = AF_INET;
 	if ((he = gethostbyname("localhost")))
 	    MEMMOVE((caddr_t)&ia.sin_addr, he->h_addr, he->h_length);
+	ia.sin_port = htons(PMAPPORT);
+#endif	/* !defined(CAN_USE_CLNT_CREATE) */
+
 	tm.tv_sec = 60;
 	tm.tv_usec = 0;
-	ia.sin_port = htons(PMAPPORT);
 /*
  * Get an RPC client handle.  Then ask for a dump of the port map.
  */
+
+#if	defined(CAN_USE_CLNT_CREATE)
+	if (!(c = clnt_create("localhost", PMAPPROG, PMAPVERS, "tcp")))
+#else	/* !defined(CAN_USE_CLNT_CREATE) */
 	if (!(c = clnttcp_create(&ia, PMAPPROG, PMAPVERS, &s, 0, 0)))
+#endif	/* defined(CAN_USE_CLNT_CREATE) */
+
 	    return;
 	if (clnt_call(c, PMAPPROC_DUMP, XDR_VOID, NULL, XDR_PMAPLIST,
 		      (caddr_t)&p, tm)
@@ -214,7 +226,7 @@ fill_portmap()
 		    cp = r->r_name;
 	    }
 	    if (!cp) {
-		(void) sprintf(buf, "%lu", p->pml_map.pm_prog);
+		(void) sprintf(buf, "%lu", (unsigned long)p->pml_map.pm_prog);
 		cp = buf;
 	    }
 	    if (!strlen(cp))
@@ -331,38 +343,67 @@ fill_porttab()
  */
 
 char *
-gethostnm(ia)
-	struct in_addr *ia;		/* pointer to Internet address */
+gethostnm(ia, af)
+	unsigned char *ia;		/* Internet address */
+	int af;				/* address family -- e.g., AF_INET
+					 * or AF_INET6 */
 {
-	char hbuf[32];
+	int al = MIN_AF_ADDR;
+	char hbuf[256];
 	static struct hostcache *hc = (struct hostcache *)NULL;
 	static int hcx = 0;
 	char *hn, *np;
-	int i;
 	struct hostent *he = (struct hostent *)NULL;
+	int i, j;
 	MALLOC_S len;
 	static int nhc = 0;
-	unsigned char *u;
-
-	u = (unsigned char *)ia;
 /*
  * Search cache.
  */
+
+#if	defined(HASIPv6)
+	if (af == AF_INET6)
+	    al = MAX_AF_ADDR;
+#endif	/* defined(HASIPv6) */
+
 	for (i = 0; i < hcx; i++) {
-	    if (u[3] == hc[i].addr[3] && u[2] == hc[i].addr[2]
-	    &&  u[1] == hc[i].addr[1] && u[0] == hc[i].addr[0])
+	    if (af != hc[i].af)
+		continue;
+	    for (j = 0; j < al; j++) {
+		if (ia[j] != hc[i].a[j])
+		    break;
+	    }
+	    if (j >= al)
 		return(hc[i].name);
 	}
 /*
- * If -n has been specified, construct a dot-form address.  Otherwise,
- * look up host name by address.  If that fails, or if there is no name
- * in the returned hostent structure, construct a dot-form version of the
- * address.
+ * If -n has been specified, construct a numeric address.  Otherwise, look up
+ * host name by address.  If that fails, or if there is no name in the returned
+ * hostent structure, construct a numeric version of the address.
  */
 	if (Fhost)
-	    he = gethostbyaddr((char *)ia, sizeof(struct in_addr), AF_INET);
+	    he = gethostbyaddr((char *)ia, al, af);
 	if (!he || !he->h_name) {
-	    (void) sprintf(hbuf, "%u.%u.%u.%u", u[0], u[1], u[2], u[3]);
+
+#if	defined(HASIPv6)
+	    if (af == AF_INET6) {
+
+	    /*
+	     * Since IPv6 numeric addresses use `:' as a separator, enclose
+	     * them in brackets.
+	     */
+		hbuf[0] = '[';
+		if (!inet_ntop(af, ia, hbuf + 1, sizeof(hbuf) - 3))
+		    (void) strcpy(&hbuf[1], "can't format IPv6 address]");
+		else
+		    (void) strcat(hbuf, "]");
+	    } else
+#endif	/* defined(HASIPv6) */
+
+	    if (af == AF_INET)
+		(void) sprintf(hbuf, "%u.%u.%u.%u", ia[0], ia[1], ia[2], ia[3]);
+	    else
+		(void) sprintf(hbuf, "(unknown AF value: %d)", af);
 	    hn = hbuf;
 	} else
 	    hn = (char *)he->h_name;
@@ -389,10 +430,10 @@ gethostnm(ia)
 		Exit(1);
 	    }
 	}
-	hc[hcx].addr[0] = u[0];
-	hc[hcx].addr[1] = u[1];
-	hc[hcx].addr[2] = u[2];
-	hc[hcx].addr[3] = u[3];
+	hc[hcx].af = af;
+	for (i = 0; i < al; i++) {
+	    hc[hcx].a[i] = ia[i];
+	}
 	hc[hcx++].name = np;
 	return(np);
 }
@@ -593,7 +634,7 @@ print_file()
 	if (PrPass && !Hdr) {
 
 	/*
-	 * Print the header line of this is the second pass and the
+	 * Print the header line if this is the second pass and the
 	 * header hasn't already been printed.
 	 */
 	    (void) printf("%-*.*s %*s", CmdColW, CmdColW, CMDTTL,
@@ -606,18 +647,34 @@ print_file()
 
 	    if (Fpgrp)
 		(void) printf(" %*s", PgrpColW, PGRPTTL);
-	    (void) printf(" %*s %*s   %*s %*s",
+	    (void) printf(" %*s %*s   %*s",
 		UserColW, USERTTL,
 		FdColW - 2, FDTTL,
-		TypeColW, TYPETTL,
-		DevColW, DEVTTL);
+		TypeColW, TYPETTL);
+
+#if	defined(HASFSTRUCT)
+	    if (Fsv) {
+		if (Fsv & FSV_FA)
+		    (void) printf(" %*s", FsColW, FSTTL);
+		if (Fsv & FSV_CT)
+		    (void) printf(" %*s", FcColW, FCTTL);
+		if (Fsv & FSV_FG)
+		    (void) printf(" %*s", FgColW, FGTTL);
+		if (Fsv & FSV_NI)
+		    (void) printf(" %*s", NiColW, NiTtl);
+	    }
+#endif	/* defined(HASFSTRUCT) */
+
+	    (void) printf(" %*s", DevColW, DEVTTL);
 	    if (Foffset)
 		(void) printf(" %*s", SzOffColW, OFFTTL);
 	    else if (Fsize)
 		(void) printf(" %*s", SzOffColW, SZTTL);
 	    else
 		(void) printf(" %*s", SzOffColW, SZOFFTTL);
-	    (void) printf(" %*s %s\n", InoColW, INOTTL, NMTTL);
+	    if (Fnlink)
+		(void) printf(" %*s", NlColW, NLTTL);
+	    (void) printf(" %*s %s\n", NodeColW, NODETTL, NMTTL);
 	    Hdr++;
 	}
 /*
@@ -625,12 +682,12 @@ print_file()
  */
 	cp = (Lp->cmd && *Lp->cmd != '\0') ? Lp->cmd : "(unknown)";
 	if (!PrPass) {
-	    if ((len = safestrlen(cp)) > CMDL)
+	    if ((len = safestrlen(cp, 2)) > CMDL)
 		len = CMDL;
 	    if (len > CmdColW)
 		CmdColW = len;
 	} else
-	    safestrprtn(cp, CmdColW, stdout, 0);
+	    safestrprtn(cp, CmdColW, stdout, 2);
 /*
  * Size or print the process ID.
  */
@@ -640,18 +697,6 @@ print_file()
 		PidColW = len;
 	} else
 	    (void) printf(" %*d", PidColW, Lp->pid);
-	if (Fpgrp) {
-
-	/*
-	 * Size or print the process group ID.
-	 */
-	    if (!PrPass) {
-		(void) sprintf(buf, "%d", Lp->pgrp);
-		if ((len = strlen(buf)) > PgrpColW)
-		    PgrpColW = len;
-	    } else
-		(void) printf(" %*d", PgrpColW, Lp->pgrp);
-	}
 
 #if	defined(HASPPID)
 	if (Fppid) {
@@ -668,6 +713,18 @@ print_file()
 	}
 #endif	/* defined(HASPPID) */
 
+	if (Fpgrp) {
+
+	/*
+	 * Size or print the process group ID.
+	 */
+	    if (!PrPass) {
+		(void) sprintf(buf, "%d", Lp->pgrp);
+		if ((len = strlen(buf)) > PgrpColW)
+		    PgrpColW = len;
+	    } else
+		(void) printf(" %*d", PgrpColW, Lp->pgrp);
+	}
 /*
  * Size or print the user ID or login name.
  */
@@ -681,12 +738,20 @@ print_file()
  * Size or print the file descriptor, access mode and lock status.
  */
 	if (!PrPass) {
-	    (void) sprintf(buf, "%s%c%c", Lf->fd, Lf->access, Lf->lock);
+	    (void) sprintf(buf, "%s%c%c",
+		Lf->fd,
+		(Lf->lock == ' ') ? Lf->access
+				  : (Lf->access == ' ') ? '-'
+							: Lf->access,
+		Lf->lock);
 	    if ((len = strlen(buf)) > FdColW)
 		FdColW = len;
 	} else
 	    (void) printf(" %*.*s%c%c", FdColW - 2, FdColW - 2, Lf->fd,
-		Lf->access, Lf->lock);
+		(Lf->lock == ' ') ? Lf->access
+				  : (Lf->access == ' ') ? '-'
+							: Lf->access,
+		Lf->lock);
 /*
  * Size or print the type.
  */
@@ -695,6 +760,57 @@ print_file()
 		TypeColW = len;
 	} else
 	    (void) printf(" %*.*s", TypeColW, TypeColW, Lf->type);
+
+#if	defined(HASFSTRUCT)
+/*
+ * Size or print the file structure address, file usage count, and node
+ * ID (address).
+ */
+
+	if (Fsv) {
+	    if (Fsv & FSV_FA) {
+		cp =  (Lf->fsv & FSV_FA) ? print_kptr(Lf->fsa, buf) : "";
+		if (!PrPass) {
+		    if ((len = strlen(cp)) > FsColW)
+			FsColW = len;
+		} else
+		    (void) printf(" %*.*s", FsColW, FsColW, cp);
+		    
+	    }
+	    if (Fsv & FSV_CT) {
+		if (Lf->fsv & FSV_CT) {
+		    (void) sprintf(buf, "%ld", Lf->fct);
+		    cp = buf;
+		} else
+		    cp = "";
+		if (!PrPass) {
+		    if ((len = strlen(cp)) > FcColW)
+			FcColW = len;
+		} else
+		    (void) printf(" %*.*s", FcColW, FcColW, cp);
+	    }
+	    if (Fsv & FSV_FG) {
+		if ((Lf->fsv & FSV_FG) && (FsvFlagX || Lf->ffg || Lf->pof))
+		    cp = print_fflags(Lf->ffg, Lf->pof);
+		else
+		    cp = "";
+		if (!PrPass) {
+		    if ((len = strlen(cp)) > FgColW)
+			FgColW = len;
+		} else
+		    (void) printf(" %*.*s", FgColW, FgColW, cp);
+	    }
+	    if (Fsv & FSV_NI) {
+		cp = (Lf->fsv & FSV_NI) ? print_kptr(Lf->fna, buf) : "";
+		if (!PrPass) {
+		    if ((len = strlen(cp)) > NiColW)
+			NiColW = len;
+		} else
+		    (void) printf(" %*.*s", NiColW, NiColW, cp);
+	    }
+	}
+#endif	/* defined(HASFSTRUCT) */
+
 /*
  * Size or print the device information.
  */
@@ -801,6 +917,21 @@ print_file()
 	    } else
 		(void) printf("%*.*s", SzOffColW, SzOffColW, "");
 	}
+/*
+ * Size or print the link count.
+ */
+	if (Fnlink) {
+	    if (Lf->nlink_def) {
+		(void) sprintf(buf, " %ld", Lf->nlink);
+		cp = buf;
+	   } else
+		cp = "";
+	    if (!PrPass) {
+		if ((len = strlen(cp)) > NlColW)
+		    NlColW = len;
+	    } else
+		(void) printf(" %*s", NlColW, cp);
+	}
 /*
  * Size or print the inode information.
  */
@@ -816,19 +947,23 @@ print_file()
 
 	    break;
 	case 2:
-	    if (Lf->iproto[0]) {
+	    if (Lf->iproto[0])
 		cp = Lf->iproto;
-		break;
-	    }
-	    /* else fall through */
+	    else
+		cp = "";
+	    break;
+	case 3:
+	    (void) sprintf(buf, "%#lx", Lf->inode);
+	    cp = buf;
+	    break;
 	default:
 	    cp = "";
 	}
 	if (!PrPass) {
-	    if ((len = strlen(cp)) > InoColW)
-		InoColW = len;
+	    if ((len = strlen(cp)) > NodeColW)
+		NodeColW = len;
 	} else {
-	    (void) printf(" %*.*s", InoColW, InoColW, cp);
+	    (void) printf(" %*.*s", NodeColW, NodeColW, cp);
 	}
 /*
  * If this is the second pass, print the name column.  (It doesn't need
@@ -864,7 +999,7 @@ printinaddr()
  * separate it from the local address with "->".
  */
 	for (i = 0, *np = '\0'; i < 2; i++) {
-	    if (!Lf->li[i].as)
+	    if (!Lf->li[i].af)
 		continue;
 	    host = port = (char *)NULL;
 	    if (i) {
@@ -887,10 +1022,22 @@ addr_too_long:
 	/*
 	 * Convert the address to a host name.
 	 */
-	    if (Lf->li[i].ia.s_addr == INADDR_ANY)
+
+#if	defined(HASIPv6)
+	    if ((Lf->li[i].af == AF_INET6
+	    &&   IN6_IS_ADDR_UNSPECIFIED(&Lf->li[i].ia.a6))
+	    ||  (Lf->li[i].af == AF_INET
+	    &&    Lf->li[i].ia.a4.s_addr == INADDR_ANY))
 		host ="*";
 	    else
-		host = gethostnm(&Lf->li[i].ia);
+		host = gethostnm((unsigned char *)&Lf->li[i].ia, Lf->li[i].af);
+#else /* !defined(HASIPv6) */
+	    if (Lf->li[i].ia.a4.s_addr == INADDR_ANY)
+		host ="*";
+	    else
+		host = gethostnm((unsigned char *)&Lf->li[i].ia, Lf->li[i].af);
+#endif	/* defined(HASIPv6) */
+
 	/*
 	 * Process the port number.
 	 */
@@ -908,12 +1055,24 @@ addr_too_long:
 		 * loopback address 127.0.0.1.  (Test 2 may not always work
 		 * -- e.g., on hosts with multiple interfaces.)
 		 */
-		    if ((src = i) && FportMap
-		    &&  (Lf->li[i].ia.s_addr == INADDR_LOOPBACK
-		    ||   (Lf->li[0].as &&
-		          Lf->li[0].ia.s_addr == Lf->li[1].ia.s_addr)))
-		    {
-			src = 0;
+		    if ((src = i) && FportMap) {
+
+#if	defined(HASIPv6)
+			if (Lf->li[0].af == AF_INET6) {
+			    if (IN6_IS_ADDR_LOOPBACK(&Lf->li[i].ia.a6)
+			    ||  IN6_ARE_ADDR_EQUAL(&Lf->li[0].ia.a6,
+						   &Lf->li[1].ia.a6)
+			    )
+				src = 0;
+			} else
+#endif	/* defined(HASIPv6) */
+
+			if (Lf->li[0].af == AF_INET) {
+			    if (Lf->li[i].ia.a4.s_addr == INADDR_LOOPBACK
+			    ||  Lf->li[0].ia.a4.s_addr == Lf->li[1].ia.a4.s_addr
+			    )
+				src = 0;
+			}
 		    }
 		    if (strcasecmp(Lf->iproto, "TCP") == 0)
 			port = lkup_port(Lf->li[i].p, 0, src);
@@ -969,8 +1128,10 @@ print_init()
 	CmdColW = strlen(CMDTTL);
 	DevColW = strlen(DEVTTL);
 	FdColW = strlen(FDTTL);
-	InoColW = strlen(INOTTL);
+	if (Fnlink)
+	    NlColW = strlen(NLTTL);
 	NmColW = strlen(NMTTL);
+	NodeColW = strlen(NODETTL);
 	PgrpColW = strlen(PGRPTTL);
 	PidColW = strlen(PIDTTL);
 	PpidColW = strlen(PPIDTTL);
@@ -982,9 +1143,18 @@ print_init()
 	    SzOffColW = strlen(SZOFFTTL);
 	TypeColW = strlen(TYPETTL);
 	UserColW = strlen(USERTTL);
+
+#if	defined(HASFSTRUCT)
+	FcColW = strlen(FCTTL);
+	FgColW = strlen(FGTTL);
+	FsColW = strlen(FSTTL);
+	NiColW = strlen(NiTtl);
+#endif	/* defined(HASFSTRUCT) */
+
 }
 
 
+#if	!defined(HASPRIVPRIPP)
 /*
  * printiproto() - print Internet protocol name
  */
@@ -997,7 +1167,7 @@ printiproto(p)
 	static int m = -1;
 	char *s;
 
-	switch(p) {
+	switch (p) {
 
 #if	defined(IPPROTO_TCP)
 	case IPPROTO_TCP:
@@ -1012,9 +1182,11 @@ printiproto(p)
 #endif	/* defined(IPPROTO_UDP) */
 
 #if	defined(IPPROTO_IP)
+# if	!defined(IPPROTO_HOPOPTS) || IPPROTO_IP!=IPPROTO_HOPOPTS
 	case IPPROTO_IP:
 	    s = "IP";
 	    break;
+# endif	/* !defined(IPPROTO_HOPOPTS) || IPPROTO_IP!=IPPROTO_HOPOPTS */
 #endif	/* defined(IPPROTO_IP) */
 
 #if	defined(IPPROTO_ICMP)
@@ -1023,6 +1195,12 @@ printiproto(p)
 	    break;
 #endif	/* defined(IPPROTO_ICMP) */
 
+#if	defined(IPPROTO_ICMPV6)
+	case IPPROTO_ICMPV6:
+	    s = "ICMPV6";
+	    break;
+#endif	/* defined(IPPROTO_ICMPV6) */
+
 #if	defined(IPPROTO_IGMP)
 	case IPPROTO_IGMP:
 	    s = "IGMP";
@@ -1089,6 +1267,530 @@ printiproto(p)
 	    break;
 #endif	/* defined(IPPROTO_RAWIF) */
 
+#if	defined(IPPROTO_HOPOPTS)
+	case IPPROTO_HOPOPTS:
+	    s = "HOPOPTS";
+	    break;
+#endif	/* defined(IPPROTO_HOPOPTS) */
+
+#if	defined(IPPROTO_IPIP)
+	case IPPROTO_IPIP:
+	    s = "IPIP";
+	    break;
+#endif	/* defined(IPPROTO_IPIP) */
+
+#if	defined(IPPROTO_ST)
+	case IPPROTO_ST:
+	    s = "ST";
+	    break;
+#endif	/* defined(IPPROTO_ST) */
+
+#if	defined(IPPROTO_PIGP)
+	case IPPROTO_PIGP:
+	    s = "PIGP";
+	    break;
+#endif	/* defined(IPPROTO_PIGP) */
+
+#if	defined(IPPROTO_RCCMON)
+	case IPPROTO_RCCMON:
+	    s = "RCCMON";
+	    break;
+#endif	/* defined(IPPROTO_RCCMON) */
+
+#if	defined(IPPROTO_NVPII)
+	case IPPROTO_NVPII:
+	    s = "NVPII";
+	    break;
+#endif	/* defined(IPPROTO_NVPII) */
+
+#if	defined(IPPROTO_ARGUS)
+	case IPPROTO_ARGUS:
+	    s = "ARGUS";
+	    break;
+#endif	/* defined(IPPROTO_ARGUS) */
+
+#if	defined(IPPROTO_EMCON)
+	case IPPROTO_EMCON:
+	    s = "EMCON";
+	    break;
+#endif	/* defined(IPPROTO_EMCON) */
+
+#if	defined(IPPROTO_XNET)
+	case IPPROTO_XNET:
+	    s = "XNET";
+	    break;
+#endif	/* defined(IPPROTO_XNET) */
+
+#if	defined(IPPROTO_CHAOS)
+	case IPPROTO_CHAOS:
+	    s = "CHAOS";
+	    break;
+#endif	/* defined(IPPROTO_CHAOS) */
+
+#if	defined(IPPROTO_MUX)
+	case IPPROTO_MUX:
+	    s = "MUX";
+	    break;
+#endif	/* defined(IPPROTO_MUX) */
+
+#if	defined(IPPROTO_MEAS)
+	case IPPROTO_MEAS:
+	    s = "MEAS";
+	    break;
+#endif	/* defined(IPPROTO_MEAS) */
+
+#if	defined(IPPROTO_HMP)
+	case IPPROTO_HMP:
+	    s = "HMP";
+	    break;
+#endif	/* defined(IPPROTO_HMP) */
+
+#if	defined(IPPROTO_PRM)
+	case IPPROTO_PRM:
+	    s = "PRM";
+	    break;
+#endif	/* defined(IPPROTO_PRM) */
+
+#if	defined(IPPROTO_TRUNK1)
+	case IPPROTO_TRUNK1:
+	    s = "TRUNK1";
+	    break;
+#endif	/* defined(IPPROTO_TRUNK1) */
+
+#if	defined(IPPROTO_TRUNK2)
+	case IPPROTO_TRUNK2:
+	    s = "TRUNK2";
+	    break;
+#endif	/* defined(IPPROTO_TRUNK2) */
+
+#if	defined(IPPROTO_LEAF1)
+	case IPPROTO_LEAF1:
+	    s = "LEAF1";
+	    break;
+#endif	/* defined(IPPROTO_LEAF1) */
+
+#if	defined(IPPROTO_LEAF2)
+	case IPPROTO_LEAF2:
+	    s = "LEAF2";
+	    break;
+#endif	/* defined(IPPROTO_LEAF2) */
+
+#if	defined(IPPROTO_RDP)
+	case IPPROTO_RDP:
+	    s = "RDP";
+	    break;
+#endif	/* defined(IPPROTO_RDP) */
+
+#if	defined(IPPROTO_IRTP)
+	case IPPROTO_IRTP:
+	    s = "IRTP";
+	    break;
+#endif	/* defined(IPPROTO_IRTP) */
+
+#if	defined(IPPROTO_TP)
+	case IPPROTO_TP:
+	    s = "TP";
+	    break;
+#endif	/* defined(IPPROTO_TP) */
+
+#if	defined(IPPROTO_BLT)
+	case IPPROTO_BLT:
+	    s = "BLT";
+	    break;
+#endif	/* defined(IPPROTO_BLT) */
+
+#if	defined(IPPROTO_NSP)
+	case IPPROTO_NSP:
+	    s = "NSP";
+	    break;
+#endif	/* defined(IPPROTO_NSP) */
+
+#if	defined(IPPROTO_INP)
+	case IPPROTO_INP:
+	    s = "INP";
+	    break;
+#endif	/* defined(IPPROTO_INP) */
+
+#if	defined(IPPROTO_SEP)
+	case IPPROTO_SEP:
+	    s = "SEP";
+	    break;
+#endif	/* defined(IPPROTO_SEP) */
+
+#if	defined(IPPROTO_3PC)
+	case IPPROTO_3PC:
+	    s = "3PC";
+	    break;
+#endif	/* defined(IPPROTO_3PC) */
+
+#if	defined(IPPROTO_IDPR)
+	case IPPROTO_IDPR:
+	    s = "IDPR";
+	    break;
+#endif	/* defined(IPPROTO_IDPR) */
+
+#if	defined(IPPROTO_XTP)
+	case IPPROTO_XTP:
+	    s = "XTP";
+	    break;
+#endif	/* defined(IPPROTO_XTP) */
+
+#if	defined(IPPROTO_DDP)
+	case IPPROTO_DDP:
+	    s = "DDP";
+	    break;
+#endif	/* defined(IPPROTO_DDP) */
+
+#if	defined(IPPROTO_CMTP)
+	case IPPROTO_CMTP:
+	    s = "CMTP";
+	    break;
+#endif	/* defined(IPPROTO_CMTP) */
+
+#if	defined(IPPROTO_TPXX)
+	case IPPROTO_TPXX:
+	    s = "TPXX";
+	    break;
+#endif	/* defined(IPPROTO_TPXX) */
+
+#if	defined(IPPROTO_IL)
+	case IPPROTO_IL:
+	    s = "IL";
+	    break;
+#endif	/* defined(IPPROTO_IL) */
+
+#if	defined(IPPROTO_IPV6)
+	case IPPROTO_IPV6:
+	    s = "IPV6";
+	    break;
+#endif	/* defined(IPPROTO_IPV6) */
+
+#if	defined(IPPROTO_SDRP)
+	case IPPROTO_SDRP:
+	    s = "SDRP";
+	    break;
+#endif	/* defined(IPPROTO_SDRP) */
+
+#if	defined(IPPROTO_ROUTING)
+	case IPPROTO_ROUTING:
+	    s = "ROUTING";
+	    break;
+#endif	/* defined(IPPROTO_ROUTING) */
+
+#if	defined(IPPROTO_FRAGMENT)
+	case IPPROTO_FRAGMENT:
+	    s = "FRAGMNT";
+	    break;
+#endif	/* defined(IPPROTO_FRAGMENT) */
+
+#if	defined(IPPROTO_IDRP)
+	case IPPROTO_IDRP:
+	    s = "IDRP";
+	    break;
+#endif	/* defined(IPPROTO_IDRP) */
+
+#if	defined(IPPROTO_RSVP)
+	case IPPROTO_RSVP:
+	    s = "RSVP";
+	    break;
+#endif	/* defined(IPPROTO_RSVP) */
+
+#if	defined(IPPROTO_GRE)
+	case IPPROTO_GRE:
+	    s = "GRE";
+	    break;
+#endif	/* defined(IPPROTO_GRE) */
+
+#if	defined(IPPROTO_MHRP)
+	case IPPROTO_MHRP:
+	    s = "MHRP";
+	    break;
+#endif	/* defined(IPPROTO_MHRP) */
+
+#if	defined(IPPROTO_BHA)
+	case IPPROTO_BHA:
+	    s = "BHA";
+	    break;
+#endif	/* defined(IPPROTO_BHA) */
+
+#if	defined(IPPROTO_ESP)
+	case IPPROTO_ESP:
+	    s = "ESP";
+	    break;
+#endif	/* defined(IPPROTO_ESP) */
+
+#if	defined(IPPROTO_AH)
+	case IPPROTO_AH:
+	    s = "AH";
+	    break;
+#endif	/* defined(IPPROTO_AH) */
+
+#if	defined(IPPROTO_INLSP)
+	case IPPROTO_INLSP:
+	    s = "INLSP";
+	    break;
+#endif	/* defined(IPPROTO_INLSP) */
+
+#if	defined(IPPROTO_SWIPE)
+	case IPPROTO_SWIPE:
+	    s = "SWIPE";
+	    break;
+#endif	/* defined(IPPROTO_SWIPE) */
+
+#if	defined(IPPROTO_NHRP)
+	case IPPROTO_NHRP:
+	    s = "NHRP";
+	    break;
+#endif	/* defined(IPPROTO_NHRP) */
+
+#if	defined(IPPROTO_NONE)
+	case IPPROTO_NONE:
+	    s = "NONE";
+	    break;
+#endif	/* defined(IPPROTO_NONE) */
+
+#if	defined(IPPROTO_DSTOPTS)
+	case IPPROTO_DSTOPTS:
+	    s = "DSTOPTS";
+	    break;
+#endif	/* defined(IPPROTO_DSTOPTS) */
+
+#if	defined(IPPROTO_AHIP)
+	case IPPROTO_AHIP:
+	    s = "AHIP";
+	    break;
+#endif	/* defined(IPPROTO_AHIP) */
+
+#if	defined(IPPROTO_CFTP)
+	case IPPROTO_CFTP:
+	    s = "CFTP";
+	    break;
+#endif	/* defined(IPPROTO_CFTP) */
+
+#if	defined(IPPROTO_SATEXPAK)
+	case IPPROTO_SATEXPAK:
+	    s = "SATEXPK";
+	    break;
+#endif	/* defined(IPPROTO_SATEXPAK) */
+
+#if	defined(IPPROTO_KRYPTOLAN)
+	case IPPROTO_KRYPTOLAN:
+	    s = "KRYPTOL";
+	    break;
+#endif	/* defined(IPPROTO_KRYPTOLAN) */
+
+#if	defined(IPPROTO_RVD)
+	case IPPROTO_RVD:
+	    s = "RVD";
+	    break;
+#endif	/* defined(IPPROTO_RVD) */
+
+#if	defined(IPPROTO_IPPC)
+	case IPPROTO_IPPC:
+	    s = "IPPC";
+	    break;
+#endif	/* defined(IPPROTO_IPPC) */
+
+#if	defined(IPPROTO_ADFS)
+	case IPPROTO_ADFS:
+	    s = "ADFS";
+	    break;
+#endif	/* defined(IPPROTO_ADFS) */
+
+#if	defined(IPPROTO_SATMON)
+	case IPPROTO_SATMON:
+	    s = "SATMON";
+	    break;
+#endif	/* defined(IPPROTO_SATMON) */
+
+#if	defined(IPPROTO_VISA)
+	case IPPROTO_VISA:
+	    s = "VISA";
+	    break;
+#endif	/* defined(IPPROTO_VISA) */
+
+#if	defined(IPPROTO_IPCV)
+	case IPPROTO_IPCV:
+	    s = "IPCV";
+	    break;
+#endif	/* defined(IPPROTO_IPCV) */
+
+#if	defined(IPPROTO_CPNX)
+	case IPPROTO_CPNX:
+	    s = "CPNX";
+	    break;
+#endif	/* defined(IPPROTO_CPNX) */
+
+#if	defined(IPPROTO_CPHB)
+	case IPPROTO_CPHB:
+	    s = "CPHB";
+	    break;
+#endif	/* defined(IPPROTO_CPHB) */
+
+#if	defined(IPPROTO_WSN)
+	case IPPROTO_WSN:
+	    s = "WSN";
+	    break;
+#endif	/* defined(IPPROTO_WSN) */
+
+#if	defined(IPPROTO_PVP)
+	case IPPROTO_PVP:
+	    s = "PVP";
+	    break;
+#endif	/* defined(IPPROTO_PVP) */
+
+#if	defined(IPPROTO_BRSATMON)
+	case IPPROTO_BRSATMON:
+	    s = "BRSATMN";
+	    break;
+#endif	/* defined(IPPROTO_BRSATMON) */
+
+#if	defined(IPPROTO_WBMON)
+	case IPPROTO_WBMON:
+	    s = "WBMON";
+	    break;
+#endif	/* defined(IPPROTO_WBMON) */
+
+#if	defined(IPPROTO_WBEXPAK)
+	case IPPROTO_WBEXPAK:
+	    s = "WBEXPAK";
+	    break;
+#endif	/* defined(IPPROTO_WBEXPAK) */
+
+#if	defined(IPPROTO_EON)
+	case IPPROTO_EON:
+	    s = "EON";
+	    break;
+#endif	/* defined(IPPROTO_EON) */
+
+#if	defined(IPPROTO_VMTP)
+	case IPPROTO_VMTP:
+	    s = "VMTP";
+	    break;
+#endif	/* defined(IPPROTO_VMTP) */
+
+#if	defined(IPPROTO_SVMTP)
+	case IPPROTO_SVMTP:
+	    s = "SVMTP";
+	    break;
+#endif	/* defined(IPPROTO_SVMTP) */
+
+#if	defined(IPPROTO_VINES)
+	case IPPROTO_VINES:
+	    s = "VINES";
+	    break;
+#endif	/* defined(IPPROTO_VINES) */
+
+#if	defined(IPPROTO_TTP)
+	case IPPROTO_TTP:
+	    s = "TTP";
+	    break;
+#endif	/* defined(IPPROTO_TTP) */
+
+#if	defined(IPPROTO_IGP)
+	case IPPROTO_IGP:
+	    s = "IGP";
+	    break;
+#endif	/* defined(IPPROTO_IGP) */
+
+#if	defined(IPPROTO_DGP)
+	case IPPROTO_DGP:
+	    s = "DGP";
+	    break;
+#endif	/* defined(IPPROTO_DGP) */
+
+#if	defined(IPPROTO_TCF)
+	case IPPROTO_TCF:
+	    s = "TCF";
+	    break;
+#endif	/* defined(IPPROTO_TCF) */
+
+#if	defined(IPPROTO_IGRP)
+	case IPPROTO_IGRP:
+	    s = "IGRP";
+	    break;
+#endif	/* defined(IPPROTO_IGRP) */
+
+#if	defined(IPPROTO_OSPFIGP)
+	case IPPROTO_OSPFIGP:
+	    s = "OSPFIGP";
+	    break;
+#endif	/* defined(IPPROTO_OSPFIGP) */
+
+#if	defined(IPPROTO_SRPC)
+	case IPPROTO_SRPC:
+	    s = "SRPC";
+	    break;
+#endif	/* defined(IPPROTO_SRPC) */
+
+#if	defined(IPPROTO_LARP)
+	case IPPROTO_LARP:
+	    s = "LARP";
+	    break;
+#endif	/* defined(IPPROTO_LARP) */
+
+#if	defined(IPPROTO_MTP)
+	case IPPROTO_MTP:
+	    s = "MTP";
+	    break;
+#endif	/* defined(IPPROTO_MTP) */
+
+#if	defined(IPPROTO_AX25)
+	case IPPROTO_AX25:
+	    s = "AX25";
+	    break;
+#endif	/* defined(IPPROTO_AX25) */
+
+#if	defined(IPPROTO_IPEIP)
+	case IPPROTO_IPEIP:
+	    s = "IPEIP";
+	    break;
+#endif	/* defined(IPPROTO_IPEIP) */
+
+#if	defined(IPPROTO_MICP)
+	case IPPROTO_MICP:
+	    s = "MICP";
+	    break;
+#endif	/* defined(IPPROTO_MICP) */
+
+#if	defined(IPPROTO_SCCSP)
+	case IPPROTO_SCCSP:
+	    s = "SCCSP";
+	    break;
+#endif	/* defined(IPPROTO_SCCSP) */
+
+#if	defined(IPPROTO_ETHERIP)
+	case IPPROTO_ETHERIP:
+	    s = "ETHERIP";
+	    break;
+#endif	/* defined(IPPROTO_ETHERIP) */
+
+#if	defined(IPPROTO_ENCAP)
+# if	!defined(IPPROTO_IPIP) || IPPROTO_IPIP!=IPPROTO_ENCAP
+	case IPPROTO_ENCAP:
+	    s = "ENCAP";
+	    break;
+# endif	/* !defined(IPPROTO_IPIP) || IPPROTO_IPIP!=IPPROTO_ENCAP */
+#endif	/* defined(IPPROTO_ENCAP) */
+
+#if	defined(IPPROTO_APES)
+	case IPPROTO_APES:
+	    s = "APES";
+	    break;
+#endif	/* defined(IPPROTO_APES) */
+
+#if	defined(IPPROTO_GMTP)
+	case IPPROTO_GMTP:
+	    s = "GMTP";
+	    break;
+#endif	/* defined(IPPROTO_GMTP) */
+
+#if	defined(IPPROTO_DIVERT)
+	case IPPROTO_DIVERT:
+	    s = "DIVERT";
+	    break;
+#endif	/* defined(IPPROTO_DIVERT) */
+
 	default:
 	    s = (char *)NULL;
 	}
@@ -1105,6 +1807,7 @@ printiproto(p)
 		(void) sprintf(Lf->iproto, "*%d?", p % (m/10));
 	}
 }
+#endif	/* !defined(HASPRIVPRIPP) */
 
 
 /*
@@ -1131,10 +1834,10 @@ printname(nl)
 	 */
 	    safestrprt(Lf->nm, stdout, 0);
 	    ps++;
-	    if (!Lf->li[0].as && !Lf->li[1].as)
+	    if (!Lf->li[0].af && !Lf->li[1].af)
 		goto print_nma;
 	}
-	if (Lf->li[0].as || Lf->li[1].as) {
+	if (Lf->li[0].af || Lf->li[1].af) {
 	    if (ps)
 		putchar(' ');
 	/*
@@ -1179,23 +1882,47 @@ printname(nl)
 	    goto print_nma;
 #endif	/* defined(HASPRIVNMCACHE) */
 
+	if (Lf->lmi_srch) {
+	    struct mounts *mp;
+	/*
+	 * Do a deferred local mount info table search for the file system
+	 * (mounted) directory name and inode number, and mounted device name.
+	 */
+	    for (mp = readmnt(); mp; mp = mp->next) {
+		if (Lf->dev == mp->dev) {
+		    Lf->fsdir = mp->dir;
+		    Lf->fsdev = mp->fsname;
+
+#if	defined(HASFSINO)
+		    Lf->fs_ino = mp->inode;
+#endif	/* defined(HASFSINO) */
+
+		    break;
+		}
+	    }
+	    Lf->lmi_srch = 0;
+	}
 	if (Lf->fsdir || Lf->fsdev) {
 
 	/*
 	 * Print the file system directory name, device name, and
 	 * possible path name components.
 	 */
+
+#if	!defined(HASNCACHE) || HASNCACHE<2
 	    if (Lf->fsdir) {
 		safestrprt(Lf->fsdir, stdout, 0);
 		ps++;
 	    }
+#endif	/* !defined(HASNCACHE) || HASNCACHE<2 */
 
 #if	defined(HASNCACHE)
+# if	HASNCACHE<2
 	    if (Lf->na
 	    &&  (cp = ncache_lookup(buf, sizeof(buf), &fp))) {
 		if (*cp == '\0')
 		    goto print_nma;
-		if (fp) {
+		if (fp && Lf->fsdir) {
 		    if (*cp != '/') {
 			cp1 = strrchr(Lf->fsdir, '/');
 			if (cp1 == (char *)NULL ||  *(cp1 + 1) != '\0')
@@ -1207,6 +1934,29 @@ printname(nl)
 		ps++;
 		goto print_nma;
 	    }
+# else	/* HASNCACHE>1 */
+	    if ((cp = ncache_lookup(buf, sizeof(buf), &fp))) {
+		if (fp) {
+		    safestrprt(cp, stdout, 0);
+		    ps++;
+		} else {
+		    if (Lf->fsdir) {
+			safestrprt(Lf->fsdir, stdout, 0);
+			ps++;
+		    }
+		    if (*cp) {
+			(void) fputs(" -- ", stdout);
+			safestrprt(cp, stdout, 0);
+			ps++;
+		    }
+		}
+		goto print_nma;
+	    }
+	    if (Lf->fsdir) {
+		safestrprt(Lf->fsdir, stdout, 0);
+		ps++;
+	    }
+# endif	/* HASNCACHE<2 */
 #endif	/* defined(HASNCACHE) */
 
 	    if (Lf->fsdev) {
@@ -1239,11 +1989,11 @@ print_nma:
 	&&  (Lf->lts.type >= 0
 
 #if	defined(HASTCPTPIQ)
-	||   Lf->lts.rqs || Lf->lts.sqs
+	||   ((Ftcptpi & TCPTPI_QUEUES) && (Lf->lts.rqs || Lf->lts.sqs))
 #endif	/* defined(HASTCPTPIQ) */
 
 #if	defined(HASTCPTPIW)
-	||   Lf->lts.rws || Lf->lts.wws
+	||   ((Ftcptpi & TCPTPI_WINDOWS) && (Lf->lts.rws || Lf->lts.wws))
 #endif	/* defined(HASTCPTPIW) */
 
 	)) {
@@ -1284,6 +2034,58 @@ printrawaddr(sa)
 }
 
 
+/*
+ * printsockty() - print socket type
+ */
+
+char *
+printsockty(ty)
+	int ty;				/* socket type -- e.g., from so_type */
+{
+	static char buf[64];
+	char *cp;
+
+	switch (ty) {
+
+#if	defined(SOCK_STREAM)
+	case SOCK_STREAM:
+	    cp = "STREAM";
+	    break;
+#endif	/* defined(SOCK_STREAM) */
+
+#if	defined(SOCK_STREAM)
+	case SOCK_DGRAM:
+	    cp = "DGRAM";
+	    break;
+#endif	/* defined(SOCK_DGRAM) */
+
+#if	defined(SOCK_RAW)
+	case SOCK_RAW:
+	    cp = "RAW";
+	    break;
+#endif	/* defined(SOCK_RAW) */
+
+#if	defined(SOCK_RDM)
+	case SOCK_RDM:
+	    cp = "RDM";
+	    break;
+#endif	/* defined(SOCK_RDM) */
+
+#if	defined(SOCK_SEQPACKET)
+	case SOCK_SEQPACKET:
+	    cp = "SEQPACKET";
+	    break;
+#endif	/* defined(SOCK_SEQPACKET) */
+
+	default:
+	    (void) sprintf(buf, "SOCK_%#x", ty);
+	    return(buf);
+	}
+	(void) sprintf(buf, "SOCK_%s", cp);
+	return(buf);
+}
+
+
 /*
  * printuid() - print User ID or login name
  */
@@ -1302,7 +2104,7 @@ printuid(u, ty)
 	static struct stat sbs;
 	static struct uidcache {
 	    uid_t u;
-	    char nm[LNML+1];
+	    char nm[LOGINML+1];
 	    struct uidcache *next;
 	} **uc = (struct uidcache **)NULL;
 	struct uidcache *up, *upn;
@@ -1345,7 +2147,7 @@ printuid(u, ty)
 		if (sbs.st_mtime != sb.st_mtime || sbs.st_ctime != sb.st_ctime)
 		{
 		    for (i = 0; i < UIDCACHEL; i++) {
-			if (!(up = uc[i])) {
+			if ((up = uc[i])) {
 			    do {
 				upn = up->next;
 				(void) free((FREE_P *)up);
@@ -1390,8 +2192,8 @@ printuid(u, ty)
 			Pn, (unsigned long)u, pw->pw_name);
 		    Exit(1);
 		}
-		(void) strncpy(upn->nm, pw->pw_name, LNML);
-		upn->nm[LNML] = '\0';
+		(void) strncpy(upn->nm, pw->pw_name, LOGINML);
+		upn->nm[LOGINML] = '\0';
 		upn->u = (uid_t)u;
 		upn->next = uc[i];
 		uc[i] = upn;
@@ -1415,8 +2217,9 @@ printuid(u, ty)
  */
 
 void
-printunkaf(fam)
+printunkaf(fam, ty)
 	int fam;			/* unknown address family */
+	int ty;				/* output type: 0 = terse; 1 = full */
 {
 	char *p, *s;
 
@@ -1441,6 +2244,12 @@ printunkaf(fam)
 	    break;
 #endif	/* defined(AF_INET) */
 
+#if	defined(AF_INET6)
+	case AF_INET6:
+	    s = "INET6";
+	    break;
+#endif	/* defined(AF_INET6) */
+
 #if	defined(AF_IMPLINK)
 	case AF_IMPLINK:
 	    s = "IMPLINK";
@@ -1705,10 +2514,17 @@ printunkaf(fam)
 #endif	/* defined(AF_USER) */
 
 	default:
-	    (void) sprintf(Namech, "no further information on family %#x", fam);
+	    if (!ty)
+		(void) sprintf(Namech, "%#x", fam);
+	    else
+		(void) sprintf(Namech,
+		    "no further information on family %#x", fam);
 	    return;
 	}
-	(void) sprintf(Namech, "no further information on %sAF_%s", p, s);
+	if (!ty)
+	    (void) sprintf(Namech, "%sAF_%s", p, s);
+	else
+	    (void) sprintf(Namech, "no further information on %sAF_%s", p, s);
 	return;
 }
 
